/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;
import org.flowlib.NewAbstractMovieClip;
/**
 * @author anssi
 */
class org.flowlib.AlphaCycleEffect {
 	private var logger:Logger;
 	
 	private var intervalId:Number = -1;
 	private var waitInterval:Number = -1;
 	private var efectActive:Boolean = false;
	private var clip:Object;
	private var fadingIn:Boolean;

	public function AlphaCycleEffect(forClip:Object) {
		logger = new Logger();
		logger.addPublisher ( new ConsolePublisher() );
		this.clip = forClip;
		this.clip.effect = this;
	}
	
	public function executeOneCycle():Void {
		clip.alphaChange = -20;
		
		if (! efectActive) {
			logger.debug("AlphaCycleEffect on " + clip._name);
			clearInterval(this.intervalId);
			this.intervalId = setInterval(this, "doCycle", 40, clip);
			logger.debug("started interval " + intervalId + " on " + clip._name);
			efectActive = true;
		}
	}
	
	public function execute(refreshRateMillis:Number):Void {
		clip.alphaChange = - 20;
		if (efectActive) return;
		logger.debug("AlphaCycleEffect, execute on " + clip._name);
		this.intervalId = setInterval(this, "doUpdateAlpha", refreshRateMillis, clip);
		efectActive = true;
	}
	
	public function terminate():Void {
		clearInterval(intervalId);
		efectActive = false;
	}
	
	private function doCycle(clip:MovieClip):Void {
		doUpdateAlpha(clip);
		
		// terminate effect after one cycle
		var alphaCycled:Boolean = clip._alpha >= 100 && clip.alphaChange > 0;
		if (alphaCycled) {
			logger.debug("clearInterval " + intervalId + " on " + clip._name);
			clearInterval(this.intervalId);
			efectActive = false;
		}
	}
	
	private function doUpdateAlpha(clip:MovieClip):Void {
//		logger.debug("doUpdateAlpha " + clip._name + " alpha = " + clip._alpha);
		if (clip._alpha >= 100 && clip.alphaChange > 0) {
			clip.alphaChange = -20;
			return;
		} else if (clip._alpha <= 40) {
			clip.alphaChange = 20;
		}
		clip._alpha = clip._alpha + clip.alphaChange;
	}
			
	private function isMouseOverClip(clip:MovieClip):Boolean {
		return clip.hitTest(_root._xmouse, _root._ymouse, true);
	}
	
	public static function fade(clip:Object, fadeIn:Boolean, alphaStep:Number):Void {
		if (fadeIn && clip._visible) return;
		if (! fadeIn && ! clip._visible) return;  
		
		var oldEffect:AlphaCycleEffect = clip.effect;
		if (oldEffect.isActive()) {
			if (fadeIn && oldEffect.fadingIn)
				return;
			if (! fadeIn && ! oldEffect.fadingIn)
				return;
			clip.effect.stop(fadeIn);
		}
		var newEffect:AlphaCycleEffect = new AlphaCycleEffect(clip);
		if (fadeIn) {
			newEffect.fadeIn(alphaStep);
		} else
			newEffect.fadeToBlack(alphaStep);
	}
	
	private function stop(makeVisible:Boolean):Void {
		clearInterval(this.intervalId);
		efectActive = false;
//		clip._alpha = makeVisible ? 100: 0;
		clip._visible = makeVisible;
	}
	
	public function isActive():Boolean {
		return efectActive;
	}
	
	private function fadeToBlack(alphaStep:Number) : Void {
		efectActive = true;
		fadingIn = false;
		clip._alpha = 100;
		logger.debug("fadeToBlack for " + clip);
		this.intervalId = setInterval(this, "doFade", 30, clip, -alphaStep  || -10);		
	}
	
	private function fadeIn(alphaStep:Number) : Void {
		efectActive = true;
		fadingIn = true;
		clip._alpha = 0;
		clip._visible = true;
		logger.debug("fadeIn for " + clip);
		this.intervalId = setInterval(this, "doFade", 30, clip, -alphaStep  || 10);		
	}

	private function doFade(clip:MovieClip, alphaStep:Number):Void {
//		NewAbstractMovieClip.logger.debug("doFade on " + clip._name + " alphaStep " + alphaStep + " current alpha " + clip._alpha);
		clip._alpha = clip._alpha + alphaStep;
		
		var done:Boolean = (alphaStep > 0 ? clip._alpha >= 100 : clip._alpha <= 0);
		if (done) {
			NewAbstractMovieClip.logger.debug("clearInterval " + intervalId + " on " + clip._name);
			stop(alphaStep > 0);
		}
	}
	
	public function show() : Void {
		if (efectActive) {
			return;
		}
		clip._alpha = 100;
		efectActive = false;
		clip.visible = true;
		clip._visible = true;
	}
//
//	private function createWait(functionAfterWait:Function) : Void {
//		logger.debug("waiting for previous effect to complete");
//		this.waitInterval = setInterval(this, "wait", 30, functionAfterWait);
//	}
//	
//	private function wait(functionAfterWait:Function):Void {
//		if (efectActive) return;
//		clearInterval(waitInterval);
//		functionAfterWait.apply(this);
//	}

}