/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.AlphaCycleEffect;
import org.flowlib.ColorFader;
import org.flowlib.DelayedFade;
import org.flowlib.EaseMove;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.ControlBar;
import org.flowplayer.FlowPlayerExternalInterface;
import org.flowplayer.MediaContext;
import org.flowplayer.MediaControl;
import org.flowplayer.MediaResizer;
import org.flowplayer.MediaSize;
import org.flowplayer.PromoUtil;
import org.flowplayer.Util;
import org.flowplayer.ads.AdServerClient;
import org.flowplayer.ads.InVideoAdController;
import org.flowplayer.ads.InVideoAdView;
import org.flowplayer.config.Config;
import org.flowplayer.email.EmailView;
import org.flowplayer.email.MailSender;
import org.flowplayer.embed.EmbedView;
import org.flowplayer.embed.FlowPlayerEmbedTag;
import org.flowplayer.image.LayeredImagesView;
import org.flowplayer.menu.FlowMenu;
import org.flowplayer.menu.FlowMenuItem;
import org.flowplayer.playlist.Clip;
import org.flowplayer.playlist.PlayList;
import org.flowplayer.thumbs.SuggestionView;
import org.flowplayer.thumbs.Thumbnail;

import flash.geom.Rectangle;

/**
 * Class that encapsulates all GUI widgest.
 */
class org.flowplayer.FLVDisplay extends NewAbstractMovieClip implements MediaContext {	
	private var videoBufferLength:Number;

	private var control:MediaControl;
	private var playList:PlayList;
	
	var seekingIntervalId:Number = -1;
	var alphaChange:Number = -20;
		
	private static var MENU_ITEM_EMBED:Number = MediaSize.maxValue() + 1;
	private static var MENU_ITEM_EMAIL:Number = MENU_ITEM_EMBED + 1;

	private static var PLAYLIST_TO_CONTROLS_GAP:Number = 0;
	private var maxControlHeight:Number = undefined;
	
	// contained movie clips
	private var mcBuffer:MovieClip;
	private var mcMenu:FlowMenu;

	private var mcFLVVideo:MovieClip;
	private var mcSuggestionView:SuggestionView;
	private var mcSWFVideo:MovieClip;
	
	private var videoFLV:Video;
	private var mcThumbList:MovieClip;
	private var mcImageView:LayeredImagesView;
	private var mcVideoAreaBorder:MovieClip;
	private var mcEmbed:EmbedView;
	private var mcEmailView:EmailView;
	public var mcFPLogo:MovieClip;
	public var mcAdView:InVideoAdView;
	private var adController:InVideoAdController;
	
	private var progressIndicatorEdgeColor:Number;
	private var progressIndicatorCenterColor:Number;
	private var videoResizer:MediaResizer;
	private var mediaViewFader:ColorFader;
	private var preFullScreenSizeOption:Number;
	private var thumbsArranged : Boolean;
	public var controlBar:ControlBar;
	public var controlBarHitArea:MovieClip;
	private var embedMenuItem : FlowMenuItem;
	private var emailMenuItem : FlowMenuItem;
	private var externalInterface : FlowPlayerExternalInterface;
	private var bufferAnimation:MovieClip;
	private var mouseMoveTimer:Number;
	
	public function init(controller:MediaControl, externalInterface:FlowPlayerExternalInterface) {
		if (controller == null) throw new Error("controller is null");
		this.control = controller;
		logger.debug("FLVDisplay:: init() with controller: " + controller);
		controller.attachVideo(mcFLVVideo.video, 'flv');
		controller.attachView(mcFLVVideo, 'flv');
		controller.attachView(mcImageView, 'jpg');
		controller.attachView(mcImageView, 'png');
		controller.attachView(mcImageView, 'overlay');
		controller.attachView(mcImageView, 'mp3');
		controller.attachView(mcSWFVideo, 'swf');
		controller.attachView(mcSuggestionView, 'suggestions');
		controller.setContext(this);
		controller.setSound(new Sound(mcFLVVideo));

		if (getConfig().enableAds()) {
			adController = new InVideoAdController(new AdServerClient(getConfig()), this.control);
			adController.setView(mcAdView);
			adController.setTracker(controlBar.tracker);
		}

		this.playList = getConfig().getPlayList();
		
		registerAsControlListener();
		controlBar.setControl(controller);
		controlBar.initStates();
		
		if (mcThumbList != undefined)
			control.addListener("onCuePointWithThumb", mcThumbList);
		
		if (mcFPLogo)
			PromoUtil.initializeLogo(mcFPLogo, this, control, getSkin().getWatermarkLinkUrl());

		var listener:Object = new Object();
		listener.display = this;
		listener.onFullScreen = function(bFull:Boolean) {
			NewAbstractMovieClip.logger.info("FlowPlayer:: onFullScreen(), " + this.display);
			if (! bFull)
				this.display.fullScreenEnded();
		};
		Stage.addListener(listener);
		this.externalInterface = externalInterface;
		controlBar.addProgressListener(externalInterface);
	}
	
	private function createChildren():Void {
		createVideoAreaBorder();
		createSWFContainer();
		createContextMenu();
		
//
//		this.onMouseMove = function():Void {
//			this.easeControlBarByMousePos();
//		};
	}

	private function createVideoAreaBorder():Void {
		mcVideoAreaBorder = createEmptyMovieClip("mcVideoAreaBorder", getNextHighestDepth());
		getSkin().drawVideoAreaBorderTo(mcVideoAreaBorder);
	}

	private function easeControlBar(makeVisible:Boolean):Void {
		if (! controlsOverVideo()) return;
		var targetPos:Number = makeVisible ? Stage.height - controlBar._height - 10 : getDisplayHeight() + (isInHWScaledFullScreen() ? 20 : 0);
		if (makeVisible) {
			// is already visible?
			if (controlBar._y < getDisplayHeight()) return;
			EaseMove.execute(controlBar, targetPos);
		} else {
			EaseMove.execute(controlBar, targetPos);
		}
	}
	
	private function createSWFContainer():Void {
		mcSWFVideo = createEmptyMovieClip("mcSWFVideo", getNextHighestDepth());
		mediaViewFader.addClip(mcSWFVideo);
	}
	
	private function getChildDescriptors():Array {
		this.visible = false;
		var descriptors:Array = [ new ClipDescriptor("mcFLVVideo", "mcFLVVideo", null) ];
		descriptors.push(new ClipDescriptor("LayeredImagesView", "mcImageView", null));
				
		logger.debug("showControls: " + getConfig().showControls());
		if (! getConfig().showControls()) {
			logger.info("Controls will be hidden");
			return descriptors;
		}
		
		descriptors.push(new ClipDescriptor("ControlBar", "controlBar", null));
		
		if (useThumbs()) {
			descriptors.push(new ClipDescriptor("ThumbnailListView", "mcThumbList", null));			
		}

		descriptors.push(new ClipDescriptor("SuggestionView", "mcSuggestionView", null));
		descriptors.push(new ClipDescriptor("EmbedView", "mcEmbed", null));
		var logoUrl:String = getSkin().getWatermarkUrl();
		if (logoUrl) {
			descriptors.push(new ClipDescriptor(null, "mcFPLogo", logoUrl));
		} else {
			var logoClipId:String = getSkin().getLogoClipId();
			if (logoClipId)
				descriptors.push(new ClipDescriptor(logoClipId, "mcFPLogo", null));
		}
		descriptors.push(new ClipDescriptor("InVideoAdView", "mcAdView", null));
		if (getConfig().getEmailPostUrl()) {
			descriptors.push(new ClipDescriptor("EmailView", "mcEmailView", null));
		}
		if (getConfig().showMenu()) {
			descriptors.push(new ClipDescriptor("FlowMenu", "mcMenu", null));
		}
		
		logger.debug("descriptors:");
		logger.debug(descriptors);
		return descriptors;
	}
		
	private function allChildrenCreated(clips:Array):Void {
		logger.debug("FLVDisplay: allChildrenCreated, child clips:");
		logger.debug(clips);
		this.mediaViewFader = new ColorFader();

		var i:Number = 0;
		logger.debug(clips);
		mcFLVVideo = clips[i++];
		
		mediaViewFader.addClip(mcFLVVideo);
		if (! mcFLVVideo)
			logger.fatal("mcFlvVideo is not defined");
		if (getConfig().useSmoothing()) {
			Video(mcFLVVideo.video).smoothing = true;
		}
		videoResizer = new MediaResizer(mcFLVVideo, getDisplayWidth(), getDisplayHeight());
		
		mcImageView = clips[i++];
		mediaViewFader.addClip(mcImageView);

		if (! getConfig().showControls()) return;		

		controlBar = clips[i++];
		controlBar.setDisplay(this);
		
		if (useThumbs()) {
			mcThumbList = clips[i++];
			mcThumbList.addEventListener("onThumbClicked", this);
			mcThumbList.addEventListener("onArranged", this);
			control.addListener("onCuePointWithThumb", mcThumbList);
			
			if (! getConfig().thumbsOnFLV() && getConfig().getThumbnails() != null) {
				mcThumbList.setThumbnails(getConfig().getThumbnails());
			}
		}

		mcSuggestionView = clips[i++];
		mcSuggestionView._visible = false;
		mcEmbed = clips[i++];
		mcEmbed._visible = false;
		mcEmbed.addEmbedCompleteListener(this);
		
		if (getSkin().getLogoClipId() || getSkin().getWatermarkUrl()) {
			mcFPLogo = clips[i++];
			var showLogo:String = getSkin().showWatermark();
			mcFPLogo._visible = (showLogo == 'always');
			if (control)
				PromoUtil.initializeLogo(mcFPLogo, this, control, getSkin().getWatermarkLinkUrl());
		}
		
		mcAdView = clips[i++];
		mcAdView.setSize(300, 70);
		mcAdView._visible = false;
		
		if (getConfig().getEmailPostUrl()) {
			mcEmailView = clips[i++];
			mcEmailView.setSize(350, 300);
			mcEmailView._visible = false;
			mcEmailView.setSender(new MailSender(getConfig().getEmailPostUrl()));
			mcEmailView.setVideoLink(getConfig().getEmailVideoLink() || 
				"http://flowplayer.org/video/2_1/share.html?config=" + FlowPlayerExternalInterface.getConfigStringForEmailLink(getConfig()));
			mcEmailView.addEmailSendingCompleteListener(this);
			onEmailSendingComplete.mcEmailView = mcEmailView;
		}
				
		if (getConfig().showMenu()) {
			mcMenu = clips[i++];
			buildMenu();
		}
		
		controlBar.initStates();
		getSkin().createDisplayCustomElements(this, getConfig());
		bufferAnimation = getSkin().createBufferAnimation(this);
		bufferAnimation._visible = false;
		
		if (! getConfig().lockControls()) {
			this.onMouseMove = function():Void {
				if (FLVDisplay(this).mouseMoveTimer)
					clearInterval(FLVDisplay(this).mouseMoveTimer);
				FLVDisplay(this).easeControlBar(true);
				FLVDisplay(this).createMouseMovetimer();
			};
			createMouseMovetimer();
		}
	}
	
	private function createMouseMovetimer():Void {
		this.mouseMoveTimer = setInterval(this, "mouseMovePausing", 4000);
	}
	
	private function mouseMovePausing():Void {
		easeControlBar(false);
	}
	
//	private function initControlBarHitArea() {
//		controlBarHitArea._visible = false;
//		
//		controlBarHitArea.beginFill(0, 20);
//		drawRectangle(controlBarHitArea, 0, 0, Stage.width, controlBar._height * 2, true);
//		controlBarHitArea.endFill();
//		
//		controlBarHitArea.onRollOut = function():Void {
//			this._parent.easeControlBar(false);
//		};
//		controlBarHitArea.onRollOver = function():Void {
//			this._parent.easeControlBar(true);
//		};
//		controlBarHitArea.onMouseMove = function():Void {
//			this._parent.easeControlBar(true);
//		};
//		controlBarHitArea.onRelease = function():Void {
//			this._parent.control.displayClicked();
//		};
//	}

	private function usePlayList():Boolean {
		var size:Number = getConfig().getPlayList().size();
		logger.debug("playlist size is " + size);
		return size > 1;
	}
	
	private function arrange():Void {
		if (! allChildrenLoaded()) return;
		if (useThumbs() && ! thumbsArranged) return;
		_root.skin._visible = false;
		if (mcThumbList) {
			NewAbstractMovieClip(mcThumbList).setSize(Stage.width, null, true);
		}
		
		mcVideoAreaBorder._x = 0;
		mcVideoAreaBorder._y = 0;
		mcVideoAreaBorder._width = getDisplayWidth() + 20;
		mcVideoAreaBorder._height = Stage.height;
		
		mcImageView.setMaxSize(getVideoAreaWidth(), getDisplayHeight());
		videoResizer.setMaxSize(getVideoAreaWidth(), getDisplayHeight());
		arrangeVideo();
		
		Util.centerClip(mcSWFVideo, getDisplayWidth(), getDisplayHeight());
		Util.centerClip(mcSuggestionView, getDisplayWidth(), getDisplayHeight());
		
		arrangeClip(mcBuffer, 5, getDisplayHeight() - 17, null, null);
		mcEmbed.setSize(getControlsWidth());

		arrangeControls();

		getSkin().arrangeDisplayCustomElements(this);
		arrangeCentered(mcEmbed, true);
		arrangeCentered(mcEmailView, true);
		arrangeCentered(mcSuggestionView, true);
		getSkin().arrangeLogo(videoResizer.originalWidth, this, isInHWScaledFullScreen());
		mcAdView._x = 5;
		mcAdView._y = 5;
		this.visible = true;
		dispatchEvent("onArranged");
	}

	public function arrangeVideo():Void {
		if (! (isInNativeFullScreen() && getConfig().useHwScaling())) {
			resizeToMenuSelection();
			Util.centerClip(mcFLVVideo, getDisplayWidth(), getDisplayHeight());
		} else {
			logger.debug("not centetring video");
		}
	}

// ========================================
		
	public function getControlsAreaXPos():Number {
		return Stage.width / 2 - getControlsWidth() / 2;
	}

	public function getControlsWidth():Number {
		return getConfig().getControlsWidth();
	}

// ========================================

	
	private function arrangeToVideoAreaMargins(clip:MovieClip):Void {
		clip._x = getSkin().getVideoAreaMargins()[1];
		clip._y = getSkin().getVideoAreaMargins()[0];
	}
	
	private function arrangeCentered(clip:NewAbstractMovieClip, centerVertically:Boolean):Void {
		clip.setSize(getConfig().getControlsWidth() - 8, getDisplayHeight());
		Util.centerClip(clip, getDisplayWidth(), centerVertically ? getDisplayHeight() : undefined);
	}
	
	public function onArranged(arrangedObject:Object):Void {
		if (arrangedObject == mcThumbList) {
			this.thumbsArranged = true;
			this.arrange();
		}
	}
	
	private function arrangeControls():Void {
		if (! getConfig().showControls()) return;
		
		controlBar.setSize(controlsOverVideo() ? getConfig().getControlsWidth() - 20 : getConfig().getControlsWidth());
		Util.centerClip(controlBar, getDisplayWidth());
		
		EaseMove.stopIfActive(controlBar);
		controlBar._y = controlsOverVideo() ? Stage.height - controlBar._height - 10 : getDisplayHeight();
		if (isInNativeFullScreen())
			EaseMove.execute(controlBar, Stage.height);
		
		getSkin().arrangeBufferAnimation(this, bufferAnimation, mcFLVVideo);
		if (mcThumbList)
			getSkin().arrangeThumbList(this, mcThumbList, controlBar);
		if (mcMenu) {
			mcMenu._x = controlBar._x + controlBar._width - mcMenu._width;
			mcMenu._y = controlBar._y - mcMenu._height;
		}
	}
	
	private function controlsOverVideo():Boolean {
		if (mcThumbList._visible) return false;
		return getConfig().controlsOverVideo();
	}

	public function getDisplayWidth():Number {
		return Stage.width;
	}
	
	public function getVideoAreaWidth():Number {
		return Stage.width - getSkin().getVideoAreaMargins()[1] * 2;
	}
	
	public function getDisplayHeight():Number {
		if (! getConfig().showControls()) return Stage.height;
		if (controlsOverVideo()) return Stage.height;
		
		var configuredHeight:Number = getConfig().getVideoHeight();
		if (configuredHeight > 0) {
			logger.info("Using configured video height " + configuredHeight);
			return configuredHeight;
		}
		var thumbListHeight:Number = mcThumbList && mcThumbList._visible ? mcThumbList._height : 0;
		return Stage.height - controlBar._height - thumbListHeight;
	}
	
	private function registerAsControlListener():Void {
		logger.debug("FLVDisplay::registering myself as control listener");
		control.addListener("onMetaData", this);
		control.addListener("onStartBuffering", this);
		control.addListener("onLoadBegin", this);
		control.addListener("onBufferFull", this);
		control.addListener("onBufferFlush", this);
		control.addListener("onPlayListChanged", this);
		control.addListener("onResume", this);
		controlBar.initStates();
	}
	
	private function startBufferingIndicator():Void {
		logger.error("enabling buffer indicator " + bufferAnimation);	
		bufferAnimation._visible = true;
		getSkin().startBufferAnimation(bufferAnimation, mcImageView);
	}

	/**
	 * Callback for controller.
	 */
	public function onStartBuffering():Void {
		NewAbstractMovieClip.logger.debug("FLVDisplay::onStartBuffering()");
		if (control.isPlaying() && ! control.isPaused())
			startBufferingIndicator();
	}

	public function onLoadBegin():Void {
		logger.debug("FLVDisplay.onLoadBegin()");
		controlBar.setEnabled(this.playList.currentClip().isControlEnabled());
	}
	
	public function onBufferFull():Void {
		logger.debug("FLVDisplay.onBufferFull()");
		stopBufferingAnimation();
		if (isInHWScaledFullScreen())
			return;
		resizeToMenuSelection();
	}
	
	public function onResume():Void {
		stopBufferingAnimation();
	}

	private function stopBufferingAnimation():Void {
		logger.debug("stopping buffering animation");
		bufferAnimation._visible = false;
		getSkin().stopBufferAnimation(bufferAnimation);
	}
		
	public function onBufferFlush():Void {
		logger.debug("onBufferFlush(), removing buffering indicator");
		stopBufferingAnimation();
	}

	public function onStatus(obj) {
		logger.info("FLVDisplay::onStatus()");

		if (obj.code.toString() == "NetStream.Play.StreamNotFound") {
			stopBufferingAnimation();
		}
	}	
	
	public function onThumbClicked(thumbNail:Thumbnail):Void {
		logger.info("seeking to " + thumbNail);
		control.seekToTime(thumbNail.getTime());
	}

	private function useThumbs():Boolean {
		return getConfig().thumbsOnFLV() || getThumbnails() != null;
	}

	private function getThumbnails():Array {
		var thumbs:Array = getConfig().getThumbnails();
		logger.info("Thumbnails specified in config:");
		logger.info(thumbs);
		return thumbs;		
	}
	
	public function onMetaData(obj):Void {
		if (isInHWScaledFullScreen())
			return;
			
		videoResizer.setOrigSize(obj.width, obj.height);

		logger.error("Original video dimensions as reported by FLV metadata are: " + obj.width +"x"+ obj.height);

		Util.centerClip(mcFLVVideo, getDisplayWidth(), getDisplayHeight());
		controlBar.enableFullScreenButton();
		if (getConfig().thumbsOnFLV())
			mcThumbList.setThumbnails(obj.cuePoints);
	}	

	public function toggleMenu():Void {
		mcMenu._visible = ! mcMenu._visible;
		if (mcMenu._visible) {
			mcMenu._alpha = 100;
			DelayedFade.execute(mcMenu, false, 10000);
		}
	}
	
	private function isMouseOverMenuArea(rollOverClip:MovieClip):Boolean {
		return rollOverClip._xmouse > rollOverClip._width - mcMenu._width && 
		rollOverClip._ymouse > rollOverClip._height - mcMenu._height &&
		rollOverClip._xmouse < rollOverClip._width &&
		rollOverClip._ymouse < rollOverClip._height; 
	}
	
	private function buildMenu() : Void {
		mcMenu.addSelectionListener(this);
		var index:Number = 0;
		var menuTexts:Array = getSkin().getMenuTexts();
		addSelectableMenuItem(index, menuTexts[index], MediaSize.FITTED_PRESERVING_ASPECT_RATIO, getConfig().getInitialMediaSize() == MediaSize.FITTED_PRESERVING_ASPECT_RATIO);
		index++;
		addSelectableMenuItem(index, menuTexts[index], MediaSize.HALF_FROM_ORIGINAL, getConfig().getInitialMediaSize() == MediaSize.HALF_FROM_ORIGINAL);
		index++;
		addSelectableMenuItem(index, menuTexts[index], MediaSize.ORIGINAL, getConfig().getInitialMediaSize() == MediaSize.ORIGINAL);
		index++;
		addSelectableMenuItem(index, menuTexts[index], MediaSize.FILLED_TO_AVAILABLE_SPACE, getConfig().getInitialMediaSize() == MediaSize.FILLED_TO_AVAILABLE_SPACE);
		index++;
		addActionMenuItem(index, menuTexts[index], MediaSize.FULL_SCREEN, getConfig().getInitialMediaSize() == MediaSize.FULL_SCREEN);
		index++;
		embedMenuItem = addActionMenuItem(index, menuTexts[index], MENU_ITEM_EMBED);
		if (getConfig().getEmailPostUrl()) {
			index++;
			emailMenuItem = addActionMenuItem(index, menuTexts[index], MENU_ITEM_EMAIL);
		}

		mcMenu.show();
		mcMenu._visible = false;
	}
	
	private function addSelectableMenuItem(itemNumber:Number, itemText:String, itemIndex:Number, makeSelected:Boolean):Void {
		if (getConfig().showMenuItem(itemNumber))
			mcMenu.addSelectableItem(itemText, itemIndex, makeSelected);
	}
	
	private function addActionMenuItem(itemNumber:Number, itemText:String, itemIndex:Number):FlowMenuItem {
		if (getConfig().showMenuItem(itemNumber))
			return mcMenu.addActionItem(itemText, itemIndex, true);
	}
	
	private function selectMenu(menuItem:Number) {
		mcMenu.selectItem(menuItem);
	}

	public function onMenuSelected(menuItem:FlowMenuItem):Void {
		if (! mcMenu) logger.error("Menu not created yet!");
		var menuItemNum:Number = Number(menuItem.object);
		if (menuItemNum == MENU_ITEM_EMBED) {
			showEmbed();
			return;
		} else if (menuItemNum == MENU_ITEM_EMAIL) {
			showEmailView();
			return;
		}
		resizeTo(Number(menuItem.object));
	}
	
	private function resizeTo(sizeOption:Number, skipCentering:Boolean, force:Boolean):Void {
		if (sizeOption == MediaSize.FULL_SCREEN) {
			controlBar.fullScreenButton.toggled();
			return;
		}

		resizeIfAllowed(playList.currentClip(), sizeOption, force);
		resizeIfAllowed(playList.peekNext(), sizeOption, force);
			
		arrangeToVideoAreaMargins(mcImageView);
		if (! skipCentering) {
			Util.centerClip(mcFLVVideo, getDisplayWidth(), getDisplayHeight());
			Util.centerClip(mcSWFVideo, getDisplayWidth(), getDisplayHeight());
		}
	}
	
	private function resizeIfAllowed(clip:Clip, sizeOption:Number, force:Boolean):Void {
				
		if (! videoResizer.hasOrigSize())
			videoResizer.setOrigSize(getDisplayWidth(), getDisplayHeight());
		
		if (clip.isVideo()) {
			logger.debug("resizing video " + clip.getURL() + ", allowed " + clip.allowResize());
			videoResizer.resizeTo(clip.allowResize() ? sizeOption : MediaSize.ORIGINAL, force);
		} else if (clip.isImage()) {
			logger.debug("resizing image " + clip.getURL());
			mcImageView.resizeTo(clip.allowResize() ? sizeOption : MediaSize.ORIGINAL, force);
		}
	}
	
	public function fullScreenInit():Void {
		preFullScreenSizeOption = videoResizer.currentSize;
		FlowPlayerExternalInterface.call("onFullScreenInit", playList.currentClip());
		getSkin().fullScreenInit(this);
		if (getConfig().useFlashNativeFullScreen()) {
			showLogo(true);
			
			if (getConfig().useHwScaling()) {
				// we need to prevent it to center the image when triggering HW scaled full-screen
				// hence we pass true here in 2nd param
				resizeTo(MediaSize.ORIGINAL, true, true);
				getSkin().arrangeLogo(videoResizer.hasOrigSize() ? videoResizer.originalWidth : Video(mcFLVVideo.video).width, this, true);
			} else
				resizeAndSetMenuSelection(MediaSize.FITTED_PRESERVING_ASPECT_RATIO);
//			resizeTo(getConfig().useHwScaling() ? MediaSize.ORIGINAL : MediaSize.FITTED_PRESERVING_ASPECT_RATIO);
			if (mcEmailView._visible) {
				setMediaViewsEnabled(true);
				AlphaCycleEffect.fade(mcEmailView, false);
			}
			enableAllPossibleMenuItems();
		}
//		easeControlBar(true);
	}
	
	public function fullScreenEnded() {
		NewAbstractMovieClip.logger.info("FlowPlayer:: fullScreenEnded()");
		FlowPlayerExternalInterface.call("onFullScreenExit", playList.currentClip());
		getConfig().setFullScreen(false);
		controlBar.disableFullScreenSettings();
		getSkin().fullScreenExit(this);
		if (getConfig().useFlashNativeFullScreen()) {
			showLogo(false);
			resizeAndSetMenuSelection(preFullScreenSizeOption);
			controlBar.fullScreenButton.release();
			enableAllPossibleMenuItems();
		}
	}
	
	private function showLogo(show:Boolean):Void {
		if (! show && getSkin().showWatermark() == 'always') return;
		if (mcFPLogo)
			mcFPLogo._visible = show;  
	}
	
	public function resizeCurrentDisplay():Void {
		resizeToMenuSelection();
	}
	
	private function resizeToMenuSelection():Void {
		if (getConfig().showMenu() && getConfig().showControls() && mcMenu.hasSelection()) {
			mcMenu.fireSelectionEvent();
		} else {
			logger.debug("menu not shown or does not have the size option -> resizing to initialScale " + getConfig().getInitialMediaSize());
			resizeTo(getConfig().getInitialMediaSize());
		}
	}
	
	private function resizeAndSetMenuSelection(sizeOption:Number):Void {
		if (getConfig().showMenu() && getConfig().showControls() && mcMenu.hasItem(sizeOption)) {
			logger.error("secting menu item " + sizeOption);
			selectMenu(sizeOption);
		} else {
			logger.error("resizing to (menu item not in use) " + sizeOption);
			resizeTo(sizeOption);
		}
	}

	public function getVideoDisplay():MovieClip {
		return mcFLVVideo;
	}

	public function getImageDisplay():MovieClip {
		return mcImageView;
	}

	public function getVideo():Video {
		return this.videoFLV;
	}

	private function showEmbed():Void {
		if (getConfig().getPlayList().currentClip().getType() == 'suggestions') return;
		logger.debug("showing embed " + mcEmbed);
		AlphaCycleEffect.fade(mcEmbed, true);
		setMediaViewsEnabled(false);
		if (! isInNativeFullScreen()) 
			enableFormOpeningItems(true);
		AlphaCycleEffect.fade(mcEmailView, false);
	}
	
	private function isInNativeFullScreen():Boolean {
		return Stage["displayState"] == "fullScreen";
	}
	
	public function isInHWScaledFullScreen():Boolean {
		return isInNativeFullScreen() && getConfig().useHwScaling();
	}

	private function showEmailView():Void {
		logger.debug("showing email " + mcEmailView);
		AlphaCycleEffect.fade(mcEmailView, true);
		setMediaViewsEnabled(false);
		enableFormOpeningItems(false);
		AlphaCycleEffect.fade(mcEmbed, false);
	}
	
	private function enableFormOpeningItems(enableEmail:Boolean):Void {
		mcMenu.enableItem(enableEmail ? emailMenuItem : embedMenuItem);
		mcMenu.disableItem(enableEmail ? embedMenuItem : emailMenuItem);
	}

	private function setMediaViewsEnabled(enabled:Boolean) : Void {
//		if (! mcImageView._visible) return;
		mcImageView.enabled = enabled;
		mcFLVVideo.enabled = enabled;
		if (! enabled)
			mediaViewFader.dim();
		else 
			mediaViewFader.brighten();
	}
	
	private function onEmailSendingComplete(sent:Object):Void {
		AlphaCycleEffect.fade(arguments.callee.mcEmailView, false);
		setMediaViewsEnabled(true);
		enableAllPossibleMenuItems();
	}

	private function onEmbedComplete():Void {
		AlphaCycleEffect.fade(mcEmbed, false);
		setMediaViewsEnabled(true);
		enableAllPossibleMenuItems();
	}
	
	private function enableAllPossibleMenuItems():Void {
		if (! isInNativeFullScreen())
			mcMenu.enableAllItems();
		else {
			mcMenu.enableItem(embedMenuItem);
			mcMenu.disableItem(emailMenuItem);
		}
	}

	public function onPlayListChanged(newPlayList:PlayList):Void {
		this.playList = newPlayList;
	}

	private function createContextMenu() : Void {
		getSkin().createContextMenu();
	}
	
	public function getVideoRectangle():Rectangle {
		mcFLVVideo.video.smoothing = false;
    	mcFLVVideo.video.deblocking = 0;
		mcFLVVideo._x = 0;
		mcFLVVideo._y = 0;
		var rect:Rectangle = new Rectangle(mcFLVVideo._x, mcFLVVideo._y, 
			videoResizer.hasOrigSize() ? videoResizer.originalWidth : Video(mcFLVVideo.video)._width,
			videoResizer.hasOrigSize() ? videoResizer.originalHeight : Video(mcFLVVideo.video)._width);
		return rect;
	}
	
	public function cleanup():Void {
		clearInterval(mouseMoveTimer);		
	}
}