/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.AlphaCycleEffect;
import org.flowplayer.ads.AdServerClient;
import org.flowplayer.ads.InVideoAd;
import org.flowplayer.ads.InVideoAdView;
import org.flowplayer.FLVDisplay;
import org.flowplayer.MediaControl;
import org.flowplayer.playlist.Clip;
import org.flowplayer.ProgressTracker;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;

class org.flowplayer.ads.InVideoAdController {
	private var logger:Logger;
	private var adView:InVideoAdView;
	private var display:FLVDisplay;
	private var adServerClient:AdServerClient;
	private var clip:Clip;
	private var control:MediaControl;
	private var adPause:Boolean;
	
	public function InVideoAdController(adServerClient:AdServerClient, control:MediaControl) {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		logger.error("Creating InVideoAdController, control = " + control);
		this.adServerClient = adServerClient;
		adServerClient.addAdsLoaderListener(this);
		control.addListener("onLoadBegin", this);
		this.control = control;
	}
	
	public function setView(adView:InVideoAdView):Void {
		this.adView = adView;
		adView._visible = false;
		adView.addClickListener(this);
		adView.addImpressionListener(this);
		adView.addMouseListener(this);
	}
	
	public function setTracker(tracker:ProgressTracker):Void {
		tracker.addProgressListener(this);
	}

	public function onLoadBegin(clip:Clip):Void {
		logger.error("InVideoAdController.onLoadBegin");
		if (! clip.getDuration()) {
			logger.info("clip does not have a duration, returning without loading ads");
			return;
		}
		this.clip = undefined;
		
		if (! clip.getURL()) return;
		if (clip.isVideo() || clip.isAudio())
			adServerClient.loadAdsFor(clip);
	}
	
	public function onAdsLoaded(clipObj:Object):Void {
		this.clip = Clip(clipObj);
		logger.debug("ads received:");
		logger.debug(clip.getAds());
	}
	
	public function onPlaybackTimeProgress(time:Number):Void {
		if (! clip.getAds()) return;
		var showingNow:Boolean = false;
		for (var i:Number = 0; i < clip.getAds().length && ! showingNow; i++) {
			var ad:InVideoAd = clip.getAds()[i];
			if (ad.isShowingAtTime(time)) {
//				logger.debug("time: "+time+", showing ad " + ad);
				showAd(ad);
				showingNow = true;
			}
		}
		if (! showingNow) {
			adView.hide();
		}
	}
	
	private function showAd(ad:InVideoAd):Void {
		if (! adView._visible)
			adView.show();
		adView.setAd(ad);
	}

	private function changeImageVisibility(makeVisible:Boolean):Void {
		if (makeVisible) {
			logger.debug("hiding in-video ad");
			AlphaCycleEffect.fade(adView, true);
		} else if (adView._visible) {
			logger.debug("showing in-video ad");
			AlphaCycleEffect.fade(adView, false);
		}
	}

	public function onAdClicked(adObj:Object):Void {
		var ad:InVideoAd = InVideoAd(adObj);
		adServerClient.sendClickEventFor(clip, ad);
		getURL(ad.getUrl());
	}
	
	public function onAdImpression(adObj:Object):Void {
		var ad:InVideoAd = InVideoAd(adObj);
		adServerClient.sendImpressionEventFor(clip, ad);
	}
	
	public function onMouseOverAd(adObj:Object):Void {
		if (control.isPlaying() && ! control.isPaused()) {
			this.adPause = true;
			this.control.pause();
		}
	}
	
	public function onMouseOutOfAd(adObj:Object):Void {
		if (! adPause) return;
		if (control.isPaused())
			control.resume();
		else
			control.playCurrent();
		adPause = false;
	}
}