/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.Util;
import org.flowplayer.ads.InVideoAd;
import org.flowplayer.skin.Skin;
import org.flowplayer.image.ImageView;
import org.flowplayer.MediaSize;
import org.flowlib.AlphaCycleEffect;

class org.flowplayer.ads.InVideoAdView extends NewAbstractMovieClip {
	
	private static var LINK_TEXT_COLOR:Number = 0xAAAAFF;
	private static var TITLE_TEXT_COLOR:Number = 0xAAAAFF;
	private static var BODY_TEXT_COLOR:Number = 0xFFFFFF;
	
	private static var EVENT_AD_CLICKED:String = "onAdClicked";
	private static var EVENT_AD_IMPRESSION:String = "onAdImpression";
	private static var EVENT_MOUSE_OVER_AD:String = "onMouseOverAd";
	private static var EVENT_MOUSE_OUT_OF_AD:String = "onMouseOutOfAd";

	private var mcBackground:MovieClip;
	private var imageView:ImageView;
	private var titleText:TextField;
	private var htmlText:TextField;
	private var linkText:TextField;
	private var btnClose:MovieClip;
	var mySkin:Skin;
	var ad:InVideoAd;
	private var closed:Boolean;

	public function InVideoAdView() {
		super();
	}
		
	public function setAd(ad:InVideoAd):Void {
		if (this.ad == ad) return;
		this.closed = false;
		logger.debug("InVideoAdView.setAd: " + ad);
		this.ad = ad;
		dispatchEvent(EVENT_AD_IMPRESSION, ad);
		setTitleText(ad.getTitle());
		setHtmlText(ad.getLine1() + "<br>" + ad.getLine2());
		setLinkText(ad.getVisibleUrl());
		
		if (ad.getImageUrl()) {
			imageView._visible = true;
			imageView.loadImage(ad.getImageUrl(), false);
		} else {
			imageView._visible = false;
		}
		arrange();
	}

	private function setTitleText(text:String):Void {
		if (! titleText)
			createTitleText();
		titleText.text = text;
	}

	private function setHtmlText(text:String):Void {
		if (! htmlText)
			createHtmlText();
		htmlText.htmlText = text;
	}
	
	private function setLinkText(text:String):Void {
		if (! text) return;
		if (! linkText)
			createLinkText();
		linkText.text = text;
	}
	
	private function createChildren():Void {
		createBackground();
		createImageView();
		createCloseButton();
		addMouseListeners();
	}
	
	private function createBackground():MovieClip {
		mcBackground = createEmptyMovieClip("mcBackground", getNextHighestDepth());
		var color:Number = 0x000000;
		mcBackground.beginFill(color, 100);
		Util.drawRoundedRectacle(mcBackground, 0, 0, 200, 50, 6);
		mcBackground.endFill();
		mcBackground._alpha = 60;
		return mcBackground;
	}

	private function createImageView():Void {
		imageView = ImageView(attachMovie("ImageView", "imageView", getNextHighestDepth()));
		imageView.addEventListener("onLoaded", this);
	}
	
	private function createCloseButton():Void {
		btnClose = getSkin().createButton(this, 15, 15, undefined, true, 0, "CloseX");
		btnClose.onRelease = function():Void {
			this._parent.closeClicked();
		};
	}

	private function createTitleText():Void {
		titleText = createTextField("titleText", getNextHighestDepth(), 0, 0, __width, 20);
		getSkin().initTextField(titleText, "left", 12, false);
		logger.debug("font is " + htmlText.getTextFormat().font);
		titleText.multiline = false;
		titleText.selectable = false;
		titleText.textColor = TITLE_TEXT_COLOR;
	}
	
	private function createLinkText():Void {
		linkText = createTextField("linkText", getNextHighestDepth(), 0, 0, __width, 20);
		getSkin().initTextField(linkText, "left", 10, true);
		linkText.multiline = false;
		linkText.selectable = false;
		linkText.textColor = LINK_TEXT_COLOR;
	}
	
	private function createHtmlText():Void {
		htmlText = createTextField("htmlText", getNextHighestDepth(), 0, 0, __width, 45);
		getSkin().initTextField(htmlText, "left", 10, false);
		logger.debug("font is " + htmlText.getTextFormat().font);
		htmlText.html = true;
		htmlText.multiline = true;
		htmlText.selectable = false;
		htmlText.textColor = BODY_TEXT_COLOR;
	}
	
	// imageView onLoaded callback
	function onLoaded():Void {
		arrange();
	}
		
	function arrange():Void {
		
		if (imageView._visible) {
			var maxImageWidth:Number = __height + 10; 
			imageView._x = 10;
			imageView.setMaxSize(maxImageWidth, __height);
			if (imageView._height > __height - 10) {
				imageView.resizeTo(MediaSize.FITTED_PRESERVING_ASPECT_RATIO);
			} else {
				imageView.resizeTo(MediaSize.ORIGINAL);
			}
			titleText._x = imageView._x  + maxImageWidth + 5;
		} else {
			titleText._x = 20;
		}
		
		titleText._y = 5;

		htmlText._x = titleText._x;
		htmlText._y = titleText._y + titleText._height;
		linkText._x = titleText._x;
		linkText._y = htmlText._y + htmlText.textHeight + 2;

		btnClose._x = __width - btnClose._width - 5;
		btnClose._y = 5;

		mcBackground._width = __width;
		mcBackground._height = __height;
	}
	
	/*
	 * This getter is just here to be configured from a testCase.
	 * Normally goes to super.getSkin().
	 */
	private function getSkin():Skin {
		if (mySkin) return mySkin;
		return super.getSkin();
	}

	private function addMouseListeners():Void {
		mcBackground.onRollOver = function() {
			NewAbstractMovieClip.logger.debug("mouseover");
			this._alpha = 90;
			MovieClip(this)._parent.onMouseOver();
		};
		mcBackground.onRollOut = function() {
			this._alpha = 60;
			MovieClip(this)._parent.onMouseOut();
		};
		mcBackground.onRelease = function() {
			MovieClip(this)._parent.onClicked();
		};
	}

	private function onClicked():Void {
		dispatchEvent(EVENT_AD_CLICKED, ad);
	}

	private function onMouseOver():Void {
		dispatchEvent(EVENT_MOUSE_OVER_AD, ad);
	}

	private function onMouseOut():Void {
		dispatchEvent(EVENT_MOUSE_OUT_OF_AD, ad);
	}
	
	public function addClickListener(listener:Object):Void {
		addEventListener(EVENT_AD_CLICKED, listener);	
	}
	
	public function addImpressionListener(listener:Object):Void {
		addEventListener(EVENT_AD_IMPRESSION, listener);	
	}
	
	public function addMouseListener(listener:Object):Void {
		addEventListener(EVENT_MOUSE_OUT_OF_AD, listener);
		addEventListener(EVENT_MOUSE_OVER_AD, listener);
	}
	
	public function show():Void {
		if (this.closed)
			return;
		AlphaCycleEffect.fade(this, true);
	}

	public function hide():Void {
		AlphaCycleEffect.fade(this, false);
	}
	
	private function closeClicked():Void {
		this.closed = true;
		hide();
	}
}