/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.image.ResizableView;
import org.flowplayer.MediaResizer;
import org.flowplayer.Util;
import org.flowlib.AlphaCycleEffect;
import org.flowlib.AbstractColorTransform;
import org.flowlib.ColorBrightener;

/**
 * @author Anssi
 */
class org.flowplayer.button.FlowButton extends NewAbstractMovieClip implements ResizableView {
	
	private var mcBackground:MovieClip;
	private var mcFace:MovieClip;
	private var maxWidth:Number;
	private var maxHeight:Number;
	private var mouseHoverTransform:AbstractColorTransform;
	private var useBackgroundHighlight:Boolean;

	private function allChildrenCreated(clips:Array):Void {
		logger.error("FlowButton. allChildrenCreated");
		this.onRollOver = function():Void {
			this.switchFaces(true);
			this.alphaCycleEffect(true);
		};
		this.onRollOut = function():Void {
			this.switchFaces(false);
			this.alphaCycleEffect(true);
		};
		createBackground();
		onAllChildrenCreated();
	}
	
	public function setButtonFaceClip(linkageId:String):Void {
		logger.error("attaching " + linkageId);
		
		mcFace = attachMovie(linkageId, "mcFace", getNextHighestDepth());

		if (isUseBackgroundHighlight()) {
	 		mcBackground.beginFill(0xFFFFFF, 50);
			Util.drawRoundedRectacle(mcBackground, 0, 0, mcFace._width+2, mcFace._height + 2, 6);
			mcBackground.endFill();
		}
		
		this._parent.hitArea = mcFace;
		if (! mcFace)
			logger.fatal("unable to attache button with id " + linkageId);
		else
			logger.error("attached button face clip " + linkageId);
	}
	
	private function createBackground():Void {
		mcBackground = createEmptyMovieClip("mcBackground", getNextHighestDepth());
		mcBackground._visible = false;
	}
	
	public function setButtonMouseHoverColorTransform(transform:AbstractColorTransform):Void {
		if (! getSkin().useAlphaChangeForButtonHover()) return;
		this.mouseHoverTransform = transform;
		mouseHoverTransform.addClip(mcFace);
	}

	public function getFaceClip():MovieClip {
		return mcFace;
	}
	
	public function switchFaces(isMouseOver:Boolean):Void {
		logger.debug(this._name + ", switchFaces()");
		
		if (getSkin().useAlphaChangeForButtonHover()) {
			if (isMouseOver)
				mouseHoverTransform.applyTransform();
			else
				mouseHoverTransform.applyNormal();
		}
		if (isUseBackgroundHighlight()) {
			logger.error("backgound highlight " + mcBackground + " " + isMouseOver);
			mcBackground._visible = isMouseOver;
		}
	}
	
	private function isUseBackgroundHighlight():Boolean {
		if (this.useBackgroundHighlight != undefined)
			return useBackgroundHighlight;
		return getSkin().useBackgroundHighlightingForButtonHover();
	}

	public function setUseBackgroundHighlight(use:Boolean):Void {
		this.useBackgroundHighlight = use;
	}

	function arrange():Void {
		if (! (mcFace)) return;
		mcBackground._width = __width;
		mcBackground._height = __height;
		
		mcFace._width = __width - 2;
		mcFace._height = __height - 2;
		mcFace._x = 1;
		mcFace._y = 1;
		
		Util.centerClip(this, maxWidth, maxHeight);
	}

	function setMaxSize(width:Number, height:Number):Void {
		this.maxWidth = width;
		this.maxHeight = height;
		arrange();
	}

	function resizeTo(sizingOption:Number):Void {
		arrange();
	}

	function loadImage(imageURL:String, allowResize:Boolean):Void {
	}

	private function onAllChildrenCreated():Void {
		// possibly overridden in subclasses
	}

	function hide() : Void {
	}

	private function applyColorEffect(mouseOver:Boolean):Void {
		if (! mouseHoverTransform) return;
		if (mouseOver)
			mouseHoverTransform.applyTransform();
		else
			mouseHoverTransform.applyNormal();
	}
	
	public function clicked():Void {
	}
}