﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.FlowPlayerExternalInterface;
import org.flowplayer.embed.FlowPlayerEmbedTag;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.playlist.PlayList;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;

/**
 * @author anssi
 */
class org.flowplayer.embed.EmbedView extends NewAbstractMovieClip {
	private static var EVENT_DONE:String = "onEmbedComplete";
	private var logger:Logger;
	private var txtMsg:TextField;
	public var txtEmbed:TextField;
	public var btnCopy:MovieClip;
	public var btnCopyLink:MovieClip;
	private var btnClose:MovieClip;
	private var mcBorder:MovieClip;
	private var txtLinkMsg:TextField;
	private var txtLink:TextField;
	private var selectInterval : Number;

	public function EmbedView() {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		selectInterval = -1;
	}

	public function addEmbedCompleteListener(listener:Object):Void {
		addEventListener(EVENT_DONE, listener);
	}
	
	private function dispatchEmbedCompleteEvent():Void {
		dispatchEvent(EVENT_DONE);
	}

	public function set markup(markup:String):Void {
		logger.error("got embed markup " + markup);
		txtEmbed.text = markup;
//		selectAll(txtEmbed);
	}
	
	public function set videoLink(videoLink:String):Void {
		logger.error("got videoLink " + videoLink);
		txtLink.text = videoLink;
//		selectAll(txtLink);
	}

	public function selectAll(text:TextField):Void {
//		if (Stage["displayState"] == "fullScreen") return;
		if (selectInterval != -1) return;
		selectInterval = setInterval(this, "doSelectAll", 50, text);
	}

	private function doSelectAll(text:TextField):Void {
		Selection.setFocus(text);
		Selection.setSelection(0, text.length);
		logger.debug("haloo2");
		clearInterval(selectInterval);
		selectInterval = -1;
	}

	private function createChildren():Void {
		mcBorder = getSkin().createEmbedBorder("mcBorder", this, 100, 50);
		getSkin().createEmbedCustomElements(this);
		
		createEmbedTextField();
		btnCopy = getSkin().createEmbedCopyButton(this);
		btnCopy.onRelease = function() {
			this._parent.copyEmbedTextToClipboard();
		};
		
		if (getConfig().getVideoLink()) {
			createLinkTextField();
			btnCopyLink = getSkin().createCopyLinkButton(this);
			btnCopyLink.onRelease = function() {
				this._parent.copyLinkTextToClipboard();
			};
		}
		
		btnClose = getSkin().createEmbedCloseButton(this);
		btnClose.onRelease = function() {
			this._parent.dispatchEmbedCompleteEvent();
		};
	}

	private function createMessage():Void {
		txtMsg = getSkin().createEmbedCodeLabel(this, "txtMsg", __width, __height);
	}

	private function createLinkMessage():Void {
		txtLinkMsg = getSkin().createEmbedLinkLabel(this, "txtLinkMsg", __width, __height); 
	}

	private function createEmbedTextField():Void {
		var markup:String = (new FlowPlayerEmbedTag(getConfig().getEmbedCode(), FlowPlayerExternalInterface.getConfigStringForEmbed(getConfig()),
			getConfig().getEmbedWidth(), getConfig().getEmbedHeight(), getConfig().useFlashNativeFullScreen())).write();
		txtEmbed = createText("txtEmbed", markup, false, false, true, __width, __height, 12);
		txtEmbed.border = false;
		txtEmbed.background = true;
		txtEmbed.textColor = 0x000000;
		txtEmbed.onSetFocus = function(oldFocus) {
			this._parent.selectAll(this);
		};
	}
	
	private function createLinkTextField() : Void {
		txtLink = createText("txtLink", getConfig().getVideoLink(), false, false, true, __width, __height, 12);
		txtLink.border = false;
		txtLink.background = true;
		txtLink.textColor = 0x000000;		
		txtLink.onSetFocus = function(oldFocus) {
			this._parent.selectAll(this);
		};
	}
	
	private function copyEmbedTextToClipboard():Void {
		copyFieldToClipBoard(txtEmbed);
	}
	
	private function copyLinkTextToClipboard():Void {
		copyFieldToClipBoard(txtLink);
	}

	private function copyFieldToClipBoard(field:TextField):Void {
		selectAll(field);
		System.setClipboard(field.text);
	}
		
	function arrange():Void {
		btnClose._x = __width - btnClose._width - 9;
		btnClose._y = 5;
		
		var MARGIN:Number = Stage.width >= 480 ? 70 : 30;
			
		if (txtMsg)
			txtMsg.removeTextField();
		createMessage();
		
		txtMsg._width = __width - 2 * MARGIN;
		txtMsg._height = 60;
		txtMsg._x = MARGIN;
		txtMsg._y = btnClose._y + btnClose._height + 5;
		
		txtEmbed._width = __width - 2*MARGIN - btnCopy._width - 5;
		txtEmbed._height = btnCopy._height;
		txtEmbed._x = MARGIN;
		txtEmbed._y = txtMsg._y + txtMsg._height + 10;

		btnCopy._y = txtEmbed._y;
		btnCopy._x = txtEmbed._x + txtEmbed._width + 5;

		if (txtLink) {
			if (txtLinkMsg)
				txtLinkMsg.removeTextField();
			createLinkMessage();
			txtLinkMsg._width = txtMsg._width;
			txtLinkMsg._height = 20;
			txtLinkMsg._x = MARGIN;
			txtLinkMsg._y = txtEmbed._y + txtEmbed._height + 10;
		
			txtLink._width = txtEmbed._width;
			txtLink._height = txtEmbed._height;
			txtLink._x = MARGIN;
			txtLink._y = txtLinkMsg._y + txtLinkMsg._height + 10;
			btnCopyLink._y = txtLink._y;
			btnCopyLink._x = txtLink._x + txtLink._width + 5;
			mcBorder._height = txtLink._y + txtLink._height + 20;
		} else {
			mcBorder._height = txtEmbed._y + txtEmbed._height + 20;
		}

				
		mcBorder._x = 0;
		mcBorder._y = 0;
		mcBorder._width = __width;
		getSkin().arrangeCustomElements(this);
	}

}