/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.MediaResizer;
import org.flowplayer.Util;
import org.flowplayer.image.ResizableView;
import org.flowlib.AlphaCycleEffect;

class org.flowplayer.image.ImageView extends NewAbstractMovieClip implements ResizableView {

	private var imageURLs:Array;
	private var imageCache:Array;
	private var resizers:Array;
	private var currentImage:MovieClip;
	private var maxWidth:Number;
	private var maxHeight:Number;
	
	public function ImageView() {
		super();
		imageURLs = new Array();
		imageCache = new Array();
		resizers = new Array();
	}
	
	public function setMaxSize(maxWidth:Number, maxHeight:Number):Void {
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;
		setMaxSizeToResizers(maxWidth, maxHeight);
		arrange();
	}
	
	private function setMaxSizeToResizers(maxWidth:Number, maxHeight:Number):Void {
		for (var key:String in resizers) {
			var resizer:MediaResizer = resizers[key];
			resizer.setMaxSize(maxWidth, maxHeight);
		}
	}
	
	public function resizeTo(sizingOption:Number):Void {
		for (var key:String in resizers) {
			var resizer:MediaResizer = resizers[key];
			resizer.resizeTo(sizingOption);
		}
		arrange();
	}
	
	public function resizeToCurrentOption():Void {
		resizeTo(undefined);
	}
	
	public function loadImage(imageURL:String, allowResize:Boolean):Void {
		if (allowResize == undefined)
			allowResize = true;
		if (isCached(imageURL)) {
			logger.debug("ImageView.loadImage:: " + imageURL + " was cached");
			show(imageURL);
		} else {
			imageURLs.push(imageURL);
			var name:String = "mcImage__" + (imageURLs.length - 1) + "--";
			if (allowResize)
				name += "resizable";
			loadClips([new ClipDescriptor(null, name, imageURL)]);
		}
	}

	private function show(imageURL:String):Void {
		for (var key:String in imageCache) {
			var image:MovieClip = imageCache[key];
			AlphaCycleEffect.fade(image, imageURL == key);
			if (image._visible) {
				this.currentImage = image;
				MediaResizer(resizers[imageURL]).resizeTo();
				arrange();
			}
			//else {
				//image._width = 0;
				//image._height = 0;
			//}
		}
		dispatchEvent("onLoaded", imageURL);
		resizeToCurrentOption();
	}
	
	public function hide():Void {
		currentImage._visible = false;
	}
	
	public function cache(clip:MovieClip):Void {
		var url:String = imageUrlFor(clip);
		imageCache[url] = clip;
		if (isResizable(clip))
			resizers[url] = new MediaResizer(clip, maxWidth, maxHeight);
	}
	
	private function imageUrlFor(clip:MovieClip):String {
		var indexPos = clip._name.indexOf("mcImage__", 0) + 9;
		var index:Number = new Number(clip._name.substring(indexPos, clip._name.indexOf("--", indexPos)));
		logger.debug("index is " + index + " in " + clip._name);
		var url = imageURLs[index];
		return url;
	}
	
	private function isResizable(clip:MovieClip):Boolean {
		return clip._name.indexOf("resizable") > 0;
	}
	
	public function isCached(imageURL:String):Boolean {
		return this.imageCache[imageURL] != undefined; 
	}

	private function loadInit(target:MovieClip) : Void {
		cache(target);
		show(imageUrlFor(target));
	}
	
	private function arrange():Void {
		if (currentImage._width == 0) return;
		Util.centerClip(currentImage, maxWidth, maxHeight);

		logger.debug("ImageView.arrange: " + getDimensions(currentImage) + ", " + imageUrlFor(currentImage));
	}

	public function scale(scalingFactor:Number):Void {
		for (var key:String in resizers) {
			var resizer:MediaResizer = resizers[key];
			resizer.scale(scalingFactor);
		}
	}

	public function resetPosition():Void {
		for (var key:String in resizers) {
			var resizer:MediaResizer = resizers[key];
			resizer.resetPosition();
		}
	}
	
	public function get imageHeight():Number {
		return currentImage._height;
	}
	
	public function get imageWidth():Number {
		return currentImage._width;
	}
	
	public function clicked():Void {
	}
}