import org.flowplayer.button.OverlayPlayButton;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.image.ResizableView;
import org.flowplayer.image.ImageView;
import org.flowplayer.MediaView;
/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
class org.flowplayer.image.LayeredImagesView extends NewAbstractMovieClip implements MediaView {
	
	private var topImage:ResizableView;
	private var underImage:ResizableView;
	private static var LOAD_EVENT_NAME:String = "onLoaded";
	private var loadedCount:Number;
	private var toLoadCount:Number;
	private var maxWidth:Number;
	private var maxHeight:Number;

	private function getChildDescriptors():Array {
		return [
			new ClipDescriptor("ImageView", "underImage", null),
			new ClipDescriptor("ImageView", "topImage", null)
		];		
	}
	
	private function allChildrenCreated(clips:Array):Void {
		underImage = clips[0];
		topImage = clips[1];
		
		this.onRollOver = function():Void {
			this.topImage.onRollOver();
		};
		this.onRollOut = function():Void {
			this.topImage.onRollOut();
		};
	}	

	function addLoadListener(listener:Object):Void {
		// FIXME: Need to cast those to ImageView in order to get the listening registered (why?)
		ImageView(topImage).addEventListener(LOAD_EVENT_NAME, this);
		ImageView(underImage).addEventListener(LOAD_EVENT_NAME, this);
		super.addEventListener(LOAD_EVENT_NAME, listener);
	}
	
	public function load(overImageUrl:String, allowOverImageResize:Boolean, underImageUrl:String, allowUnderImageResize:Boolean) {
		loadedCount = 0;
		toLoadCount = 0;
		loadTo(underImage, underImageUrl, allowUnderImageResize);
		if (overImageUrl)
			loadTo(topImage, overImageUrl, allowOverImageResize);
		else
			removeTopImage();
	}
	
	public function hideUnderImage():Void {
		underImage.hide();
	}
	
	public function hideTopImage():Void {
		topImage.hide();
	}
		
	public function setTopClip(topImage:ResizableView):Void {
		this.topImage = topImage;
		topImage.setMaxSize(this.maxWidth, this.maxHeight);
	}

	private function removeTopImage():Void {
		MovieClip(this.topImage).removeMovieClip();
	}

	private function loadTo(view:ResizableView, url:String, allowResize:Boolean) {
		if (! url) return;
		toLoadCount++;
		view.loadImage(url, allowResize);
	}
	
	public function onLoaded(url):Void {
		if (++loadedCount == toLoadCount)
			dispatchEvent(LOAD_EVENT_NAME); 
	}
	
	private function arrange():Void {
		topImage.setSize(this.__width, this.__height);
		underImage.setSize(this.__width, this.__height);
//		this.__width = Math.max(MovieClip(topImage).__width, MovieClip(underImage).__width);
//		this.__height = Math.max(MovieClip(topImage).__height, MovieClip(underImage).__height);
	}	
		
	public function resizeTo(sizingOption:Number):Void {
		topImage.resizeTo(sizingOption);
		underImage.resizeTo(sizingOption);	
	}
	
	public function setMaxSize(width:Number, height:Number):Void {
		this.maxWidth = width;
		this.maxHeight = height;
		topImage.setMaxSize(width, height);
		underImage.setMaxSize(width, height);
	}

	function handlesOnRelease():Boolean {
		return false;
	}
	
	public function topViewClicked():Void {
		// TODO: de-hack this
		if (topImage instanceof OverlayPlayButton)
			OverlayPlayButton(topImage).playClicked();
	}
	
}