﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.button.OverlayPlayButton;

import flash.filters.GlowFilter;

import org.flowlib.AbstractSlider;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.ClipDescriptor;
import org.flowplayer.ControlBar;
import org.flowplayer.FLVDisplay;
import org.flowplayer.Util;
import org.flowplayer.animation.Rotation;
import org.flowplayer.button.FlowButton;
import org.flowplayer.config.Config;
import org.flowplayer.embed.EmbedView;
import org.flowplayer.image.ResizableView;
import org.flowplayer.skin.AirDarkSkin;
import org.flowplayer.skin.AirLightSkin;
import org.flowplayer.skin.GraySkin;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;

/**
 * @author anssi
 */
class org.flowplayer.skin.Skin {

	private static var PLAYLIST_TO_CONTROLS_GAP:Number = 0;
	private var skinId : String;
	private var config:Config;
	private var logger:Logger;	
	private static var instance:Skin;
	
	public static function create(config:Config, skinId:String):Skin {
		if (instance == null) {
			if (skinId == 'airlight')
				instance = new AirLightSkin(config);
			else if (skinId == 'airdark')
				instance = new AirDarkSkin(config);
			else if (skinId == 'gray')
				instance = new GraySkin(config);
			else
				instance = new GraySkin(config);
		}
		return instance;
		
	}
	
	public function useAlphaChangeForButtonHover():Boolean {
		return true;
	}
	
	public function useBackgroundHighlightingForButtonHover():Boolean {
		return true;
	}
	
	public function getControlBarBackgroundColor():Number {
		if (config.getControlBarBackgroundColor() != undefined)
			return config.getControlBarBackgroundColor();
		return 0x888888;
	}
	
	public function toString():String {
		return "Skin: '" + this.skinId + "'";
	}
		
	public function createDisplayCustomElements(display:FLVDisplay, config:Config):Void {
	}
		
	public function createContextMenu():Void {
		var menu:ContextMenu = new ContextMenu();
		menu.hideBuiltInItems();
		menu.customItems = [ 
			new ContextMenuItem("FlowPlayer 2.1.2", function() {}), 
			new ContextMenuItem("Visit flowplayer.org",
				function() { getURL("http://flowplayer.org"); }),
			new ContextMenuItem("Report a bug",
				function() { getURL("http://sourceforge.net/tracker/?func=add&group_id=133868&atid=728380"); })
		];
		_root.menu = menu;
	}
	public function getWatermarkUrl():String {
		return undefined;
	}
	public function showWatermark():String {
		return "fullscreen";
	}
	public function getWatermarkLinkUrl():String {
		return "http://flowplayer.org";
	}
	public function getLogoClipId():String {
		return "FlowPlayerLogo";
	}

	private function Skin(config:Config, skinId:String) {
		this.config = config;
		this.skinId = skinId;
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
		logger.debug("created skin with id '" + skinId + "'");
	}
		
	private function drawControlBarBackgroundTo(mcBack:MovieClip, drawRoundedAndSemiTransparent:Boolean, bar:ControlBar):Void {
		mcBack.clear();
		
		var color:Number = getControlBarBackgroundColor();
		var alpha:Number;
		if (color == -1)
			alpha = 0;
		else
			alpha = drawRoundedAndSemiTransparent ? 50 : 100;
			
		var colors:Array = [0xffffff, 0x929293, 0x060606];
		var matrix:Object = { matrixType:"box", x:0, y:0, w:Stage.width, h:controlBarHeight(bar), r:Math.PI/2 };
		
		mcBack.beginFill(color, alpha);
		if (drawRoundedAndSemiTransparent)
			Util.drawRoundedRectacle(mcBack, 0, 0, bar.getOriginalWidth() || Stage.width - 100,  controlBarHeight(bar), 6);
		else
			Util.drawRectangle(mcBack, 0, 0, bar.getOriginalWidth() || Stage.width, controlBarHeight(bar), false);
		mcBack.endFill();

		if (config.getControlBarGloss() != "none") {
			mcBack.beginGradientFill("linear", 
				colors, 
				config.getControlBarGloss() == "low" ? [33, 10, 10] : [67, 21, 21], [0, 128, 255], matrix);
			if (drawRoundedAndSemiTransparent)
				Util.drawRoundedRectacle(mcBack, 0, 0, bar.getOriginalWidth() || Stage.width - 100,  controlBarHeight(bar), 6);
			else
				Util.drawRectangle(mcBack, 0, 0, bar.getOriginalWidth() || Stage.width, controlBarHeight(bar), false);
			mcBack.endFill();
		}
	}

	private function controlBarHeight(bar:ControlBar):Number {
		var margins:Array = getControlAreaMargins();
		return bar.maxWidgetHeight() + margins[0] + margins[2];
	}
	
	private function volumeBarHeight(bar:ControlBar):Number {
		return bar.maxWidgetHeight();
	}

	public function drawProgressBarBorderTo(mcBorder:MovieClip):Void {
		beginGradientFill(mcBorder, Stage.width, 20, getProgressBarBorderColor1(), getProgressBarBorderColor2(), 34);
		Util.drawRoundedRectacle(mcBorder, 0, 0, Stage.width, 20, 8);
		mcBorder.endFill();
	}

	public function onCreateProgressTrackerValue2Mc(mcValue2:MovieClip):Void {
		beginGradientFill(mcValue2, Stage.width, 20, getBufferIndicatorColor1(), getBufferIndicatorColor2(), 50);
		Util.drawRoundedRectacle(mcValue2, 0, 0, Stage.width, 20, 8);
		mcValue2.endFill();
	}
	
	public function drawVideoAreaBorderTo(mcBorder:MovieClip):Void {
		mcBorder.beginFill(0);
		Util.drawRectangle(mcBorder, 100, 14, 0, 0, false);
		mcBorder.endFill();
	}
	
	public function drawVolumeBarBorderTo(mcBorder:MovieClip):Boolean {
		beginGradientFill(mcBorder, Stage.width, 20, getProgressBarBorderColor1(), getProgressBarBorderColor2(), 34);
		Util.drawRoundedRectacle(mcBorder, 0, 0, 70, 20, 8);
		mcBorder.endFill();
		
		// return false => do not draw the slider triangle
		return false;
	}
	
	public function onCreateProgressTrackerValueMc(mcValue:MovieClip):Void {
		// create invisible bar
		drawRectangleWithGradientFill(mcValue, 100, 20, getProgressIndicatorColor1(), getProgressIndicatorColor2(), false, 0);
	}
	
	public function onCreateVolumeValueMc(mcValue:MovieClip, slider:AbstractSlider):Void {
		drawRectangleWithGradientFill(mcValue, 100, 20, getProgressIndicatorColor1(), getProgressIndicatorColor2(), false);
		mcValue._visible = false;
	}
	
	public function onCreateVolumeValue2Mc(mcValue2:MovieClip, slider:AbstractSlider):Void {
		drawRectangleWithGradientFill(mcValue2, 100, 20, getBufferIndicatorColor1(), getBufferIndicatorColor2(), false);
		mcValue2._visible = false;
	}

	public function drawProgressBarBackgroundTo(mcBack:MovieClip):Void {
	}
	
	public function drawVolumeBarBackgroundTo(mcBorder:MovieClip):Void {
	}
	
	public function getVolumeSliderChildDescriptors():Array {
		return [
			new ClipDescriptor("Dragger", "mcDragger", null)
		];		
	}
	
	private function drawRectangleWithGradientFill(clip:MovieClip, width:Number, height:Number, color1:Number, color2:Number, drawBorder:Boolean, alpha:Number):Void {
		beginGradientFill(clip, width, height, color1, color2, alpha);
		Util.drawRectangle(clip, 0, 0, 100, 20, drawBorder);
		clip.endFill();
	}
	
	private function beginGradientFill(clip:MovieClip, width:Number, height:Number, color1:Number, color2:Number, alpha:Number):Void {
		var colors:Array = [color1, color2, color1];
		var matrix:Object = { matrixType:"box", x:0, y:0, w:width, h:height, r:Math.PI/2 };
		var alphaVal:Number = alpha == undefined ? 100 : alpha;
		
		clip.beginGradientFill("linear", 
			colors, 
			[alphaVal, alphaVal, alphaVal], [0, 128, 255], matrix);
	}
	
	private function drawRectangleWithGradientFill2(clip:MovieClip, width:Number, height:Number, color1:Number, color2:Number, drawBorder:Boolean, borderRGB:Number):Void {
		var colors:Array = [color1, color2];
		var matrix:Object = { matrixType:"box", x:0, y:0, w:width, h:height, r:Math.PI/2 };
		
		clip.beginGradientFill("linear", 
			colors, 
			[100, 100], [0, 255], matrix);

		Util.drawRectangle(clip, 0, 0, width, height, drawBorder, borderRGB);
		clip.endFill();
	}
		 
 	public function getProgressIndicatorColor1():Number {
 		if (config.getProgressIndicatorColor1() != undefined)
 			return config.getProgressIndicatorColor1();
 		return 0x222222;
 	}
 	
 	public function getProgressIndicatorColor2():Number {
 		if (config.getProgressIndicatorColor2() != undefined)
 			return config.getProgressIndicatorColor2();
 		return  0x222222;
 	}
 	
 	public function getBufferIndicatorColor1():Number {
 		if (config.getBufferIndicatorColor1() != undefined)
 			return config.getBufferIndicatorColor1();
 		return 0x000000;
 	}
 	
 	public function getBufferIndicatorColor2():Number {
 		if (config.getBufferIndicatorColor2() != undefined)
 			return config.getBufferIndicatorColor2();
 		return 0x555555;
 	}
 	
 	public function getVolumeBarTriangleColor():Number {
 		return 0x444444;
 	}
 	
 	public function getProgressBarBorderColor1():Number {
 		if (config.getProgressBarBorderColor1() != undefined)
 		 return config.getProgressBarBorderColor1();
 		return 0;
 	}
 	
 	public function getProgressBarBorderColor2():Number {
 		if (config.getProgressBarBorderColor2() != undefined)
 			return config.getProgressBarBorderColor2();
 		return 0;
 	}
	
	public function getBufferingAnimationColor():Number {
		if (config.getBufferingAnimationColor() != undefined)
			return config.getBufferingAnimationColor();
 		return 0x333333;
// Following makes it impossible to pass pure black color (value 0)
// 		return config.getBufferingAnimationColor() || 0x333333;
 	}
	
	public function getTimeDisplayFontColor():Number {
		if (config.getTimeDisplayFontColor() != undefined)
			return config.getTimeDisplayFontColor();
		return 0xFF0000;
	}
	
	public function getSpaceAfterWidget(widget:MovieClip):Number {
		var space:Number = 8;
		
		if (widget._name == 'menuButton')
			space = 0;
		else if (widget._name == 'fullScreenButton')
			space = 0;
		else if (widget._name == 'volume')
			space = 8;
		else if (widget._name == 'volumeIcon')
			space = 4;
		else if (widget._name == 'playButton')
			space = 4;
		else if (widget._name == 'pauseButton')
			space = 4;
		else if (widget._name == 'stopButton')
			space = 4;
		else if (widget._name == 'pauseButton')
			space = 4;
		else if (widget._name == 'nextButton')
			space = 4;
		else if (widget._name == 'prevButton')
			space = 4;
		else if (widget._name == 'tracker')
			space = 8;
		logger.debug("widget name " + widget._name + ", space after " + space);
		return space;
	}
	
	public function getProgressBarXPos():Number {
		return 0;
	}
	
	public function getProgressBarToRightEdgeGap():Number {
		return 0;
	}

	public function getVolumeSliderWidth() : Number {
		return 40;
	}

	public function getTimeDisplayFontSize():Number {
		return 10;
	}

	private function arrangeSliderDefault(slider:AbstractSlider):Void {
//		slider.mcLabel._width = 80;
		var barHeight:Number = slider.barHeight;
		var barXPos:Number = slider.barXPos;
		
//		Util.arrangeClip(slider.mcBackground, 0, 0, slider.__width - 1, barHeight);
		Util.arrangeClip(slider.mcBorder, barXPos, 0, slider.__width - barXPos - (slider.mcLabel._visible ? slider.mcLabel._width : 0), barHeight);
		slider.mcBackground._width = slider.mcBorder._width;
		slider.mcBackground._height = slider.mcBorder._height;
	 	
		slider.mcValue._height = barHeight;
		slider.mcValue2._height = barHeight;
		slider.mcDragger._height = barHeight;

		slider.mcValue._x = barXPos;
		slider.mcValue2._x = barXPos;
		slider.mcValue._width = 0;
		slider.mcValue2._width = 0;
		slider.mcDragger._x = barXPos;
		
		Util.centerClip(slider.mcBorder, null, barHeight);
		Util.centerClip(slider.mcValue, null, barHeight);
		Util.centerClip(slider.mcValue2, null, barHeight);
		Util.centerClip(slider.mcDragger, null, barHeight);
//		Util.centerClip(slider.mcLabel, null, barHeight);
		slider.mcLabel._y = -2;
		if (slider.mcLabel) {
			slider.mcLabel._x = slider.mcBorder._width + getSpaceAfterWidget();
		}
//		slider.mcLabel._y = centeredY(slider.mcLabel, slider.mcDragger);
	}

	public function arrangeProgressTracker(tracker:AbstractSlider) : Void {
		arrangeSliderDefault(tracker);
	}
	
	public function arrangeVolumeSlider(slider:AbstractSlider):Void {
		arrangeSliderDefault(slider);
	}
	
	public function createEmbedCopyButton(parent:MovieClip):MovieClip {
		return createButton(parent, 60, 20, "Copy");
	}
	
	public function createCopyLinkButton(parent:MovieClip):MovieClip {
		return createButton(parent, 60, 20, "Copy");
	}
	
	public function createEmbedCloseButton(parent:MovieClip):MovieClip {
		return createButton(parent, 15, 15, undefined, true, 12, "CloseX");
	}
	
	public function createEmailCloseButton(parent:MovieClip):MovieClip {
		return createButton(parent, 15, 15, undefined, true, 12, "CloseX");
	}
	
	public function createEmbedCodeLabel(parent:MovieClip, id:String, width:Number, height:Number):TextField {
		var text:String = "You can embed this clip on your site.\nJust click the copy button and paste\nthe code below to your page.";
		return Util.createText(parent, this, id, text, true, true, false, width, height, Stage.width >= 350 ? 12 : 10);
	}
	
	public function createEmbedLinkLabel(parent:MovieClip, id:String, width:Number, height:Number):TextField {
		return Util.createText(parent, this, id, "Direct link to this file:", false, false, false, width, height, Stage.width >= 350 ? 12 : 10);
	}
	
	public function createEmailCancelButton(parent:MovieClip):MovieClip {
		return createButton(parent, 50, 20, "Cancel", true, 12);
	}
	
	public function createEmailPostButton(parent:MovieClip):MovieClip {
		return createButton(parent, 50, 20, "Send", false, 12);
	}

	public function createEmailViewMainTitle(parent:MovieClip, id:String, width:Number, height:Number):TextField {
		return Util.createText(parent, this, id, "Send a link to this video to your friend", true, true, false, width, height);
	}

	public function createEmailViewFromLabel(parent:MovieClip, id:String, width:Number, height:Number):TextField {
		return Util.createText(parent, this, id, "From:", true, true, false, width, height);
	}
	
	public function createEmailViewMessageBodyLabel(parent:MovieClip, id:String, width:Number, height:Number):TextField {
		return Util.createText(parent, this, id, "Message:", true, true, false, width, height);		
	}
	
	public function createEmailViewToLabel(parent:MovieClip, id:String, width:Number, height:Number):TextField {
		return Util.createText(parent, this, id, "To e-mail:", false, false, false, width, height);		
	}
	
	public function createEmailViewToExtraLabel(parent:MovieClip, id:String, width:Number, height:Number):TextField {
		return Util.createText(parent, this, id, "Separate multiple addesses with commas.", false, false, false, width, height);		
	}

	public function createReplayButton(parent:MovieClip):MovieClip {
		return createButton(parent, 70, 20, "Replay", false, 12);
	}
		
	public function createForwardButton(parent:MovieClip):MovieClip {
		return createButton(parent, 20, 20, ">", false, 12);
	}
		
	public function createBackwardButton(parent:MovieClip):MovieClip {
		return createButton(parent, 20, 20, "<", false, 12);
	}
	
	public function createButton(parent:MovieClip, btnWidth:Number, btnHeight:Number, buttonText:String, useRedColor:Boolean, textSize:Number, iconId:String):MovieClip {
		var button:MovieClip = parent.createEmptyMovieClip("button", parent.getNextHighestDepth());
		
		var fillType:String = "linear";
//		var colors:Array = useRedColor ? [0xFAD4DB, 0xEC748B, 0xC13A59, 0xA81230] : [0xFFFFFF, 0xAAAAAA, 0x000000, 0x000000];
		var colors:Array = useRedColor ? [0xFAD4DB, 0xEC748B, 0xC13A59, 0xA81230] : [0xAAAAAA, 0x555555, 0x333333, 0x333333];
		var alphas:Array = [100, 100, 100, 100];
		var ratios:Array = [0, 126, 127, 255];
		var matrix:Object = {matrixType:"box", x:0, y:0, w:btnWidth, h:btnHeight, r:90/180*Math.PI};
		
		button.createEmptyMovieClip("buttonBkg", button.getNextHighestDepth());
		button.buttonBkg.lineStyle(0, useRedColor ? 0x820F26 : 0x333333, btnWidth, true, "none", "square", "round");
		button.buttonBkg.beginGradientFill(fillType, colors, alphas, ratios, matrix);
		button.buttonBkg.lineTo(btnWidth, 0);
		button.buttonBkg.lineTo(btnWidth, btnHeight);
		button.buttonBkg.lineTo(0, btnHeight);
		button.buttonBkg.lineTo(0, 0);
		button.buttonBkg.endFill();
		
		if (iconId) {
			button.attachMovie(iconId, "icon", button.getNextHighestDepth());
			button.icon._height = btnHeight - 6;
			button.icon._width = button.icon._height;
			button.icon._y = 3;
			button.icon._x = 3;
		}
		
		if (buttonText) {
			var textFieldWidth:Number = button.icon? btnWidth - button.icon._width : btnWidth;
			button.createTextField("labelText", button.getNextHighestDepth(), 0, 2, textFieldWidth, btnHeight);
			initTextField(button.labelText, "center", textSize || 12);
			button.labelText.textColor = 0xFFFFFF;
			button.labelText.selectable = false;
			var labelGlow:GlowFilter = new GlowFilter(0xFFFFFF, .30, 4, 4, 3, 3);
			var labelFilters:Array = [labelGlow];
			button.labelText.filters = labelFilters;
			button.labelText.text = buttonText;
			button.labelText._x = button.icon ? button.icon._width : 0;
		}
		
		return button;
	}
	
	public function initTextField(field:TextField, align:String, size:Number, bold:Boolean, fonts:String):TextFormat {
		var format:TextFormat = getDefaultTextFormat(size, fonts);
//		field.embedFonts = true;
//		field.antiAliasType = "advanced";
		if (align)
			format.align = align;
		if (bold)
			format.bold = bold;
		if (size)
			format.size = size;
		field.setNewTextFormat(format);
		field.setTextFormat(0, field.text.length, format);
		return format;
	}

	public function getDefaultTextFormat(fontSize:Number, fonts:String):TextFormat {
		var txtFormat:TextFormat = new TextFormat();
		txtFormat.size = fontSize;
		txtFormat.font = fonts || "Verdana, Sans, FreeSans, Bitstream Vera";
		return txtFormat;
	}

	public function createEmbedBorder(id:String, parent:NewAbstractMovieClip, width:Number, height:Number):MovieClip {
		var clip:MovieClip = parent.createEmptyMovieClip(id, parent.getNextHighestDepth());
		clip.beginFill(0x333333, 50);
		Util.drawRectangleWithBorder(clip, 0, 0, width, height);
		clip.endFill();
		return clip;
	}

	public function createEmailViewBackground(id:String, parent:NewAbstractMovieClip, widht:Number, height:Number):MovieClip {
		return createEmbedBorder(id, parent, widht, height);
	}

	public function createEmbedCustomElements(parent:EmbedView):Void {
	}

	public function arrangeCustomElements(parent:EmbedView):Void {
		
	}

	public function createBufferAnimation(display:FLVDisplay):MovieClip {
		return display.attachMovie("BufferingAnimation", "bufferAnimation", display.getNextHighestDepth());
	}
	
	public function startBufferAnimation(animation:MovieClip, mcImageView:MovieClip):Void {
		Rotation(animation).start();
	}
	
	public function stopBufferAnimation(animation:MovieClip):Void {
		Rotation(animation).stop();
	}
	
	public function arrangeBufferAnimation(display:FLVDisplay, animation:MovieClip, scaleReferenceClip:MovieClip):Void {
//		Util.centerClip(animation, display.getDisplayWidth(), display.getDisplayHeight());
		animation._x = display.getDisplayWidth() / 2;
		animation._y = display.getDisplayHeight() / 2;
	}

	public function arrangeDisplayCustomElements(display:FLVDisplay):Void {
	}
	
	public function getOverlayClip(parent:MovieClip, overlayId:String):ResizableView {
//		logger.error("Skin.getOverlayClip(), overlayId " + overlayId + ", parent " + parent);
		if (overlayId != 'play') return undefined;
		if (parent["overlay_" + overlayId]) {
			return parent["overlay_" + overlayId];
		}
		var overlay:ResizableView = ResizableView(parent.attachMovie("OverlayPlayButton", "overlay_" + overlayId, parent.getNextHighestDepth()));
		return overlay;
	}

	public function showButtonEffects():Boolean {
		return true;
	}

	public function getMenuTexts():Array {
		return [ "Fit to window", "Half size", "Original size", "Fill window", "Full screen", "Embed...", "E-mail to a friend..." ];
	}
	
	public function getMenuItemBorderAlpha():Number {
		return 60;
	}

//	public function getProgressDraggerGripPos(slider:AbstractSlider):Number {
//		return slider.mcDragger._xmouse - slider.mcDragger._width;
//	}
	
	/*
	 * Control bar arranging: 
	 */
	
	public function arrangeControlBar(bar:ControlBar):Void {
		drawControlBarBackgroundTo(bar.mcBackground, config.controlsOverVideo(), bar);
		arrangeControlBarBackground(bar);
		var leftEdge:Number = arrangeLeftEdgeControls(bar);
		arrangeRightEdgeControls(bar, leftEdge);		
	}
	
	private function arrangeControlBarBackground(bar:ControlBar) : Void {
		bar.mcBackground._x = 0;
		bar.mcBackground._y = 0;
		bar.mcBackground._width = bar.__width;
//		bar.border._height = bar._height;
		bar.mcBackground._height = controlBarHeight(bar);
	}
	
	private function arrangeLeftEdgeControls(bar:ControlBar):Number {
		var leftEdge:Number = getControlAreaMargins()[3];
		leftEdge = arrangeStopButton(bar, leftEdge);
		leftEdge = arrangePlayButton(bar, leftEdge);
		leftEdge = arrangePauseButton(bar, leftEdge);
		leftEdge = arrangePrevButton(bar, leftEdge);
		leftEdge = arrangeNextButton(bar, leftEdge);
		return leftEdge;
	}
	
	private function arrangeRightEdgeControls(bar:ControlBar, leftEdge:Number):Void {
		var edge:Number =  config.showScrubber() ? (bar.__width - getControlAreaMargins()[1]) : leftEdge;
		edge = arrangeMenuButton(bar, edge);
		edge = arrangeFullScreenButton(bar, edge);
		edge = arrangeVolumeControl(bar, edge);
		edge = arrangeMuteVolumeButton(bar, edge);
		edge = arrangeProgressControl(bar, edge);
	}
	
	private function arrangeStopButton(bar:ControlBar, leftEdge:Number):Number {
		if (! bar.showStopButton()) return leftEdge;
		return arrangeToLeftEdge(leftEdge, getSpaceAfterWidget(bar.stopButton), bar.stopButton, bar);
	}
	
	private function arrangePlayButton(bar:ControlBar, leftEdge:Number):Number {
		if (! bar.showPlayButton()) return leftEdge;
		return arrangeToLeftEdge(leftEdge, getSpaceAfterWidget(bar.playButton), bar.playButton, bar);
	}
	
	private function arrangePauseButton(bar:ControlBar, leftEdge:Number):Number {
		if (! bar.showPauseButton()) return leftEdge;
		if (bar.showPlayButton()) {
			bar.pauseButton._x = bar.playButton._x;
			bar.pauseButton._y = bar.playButton._y;
		} else {
			return arrangeToLeftEdge(leftEdge, getSpaceAfterWidget(bar.pauseButton), bar.pauseButton, bar);
		}
		return leftEdge;
	}

	private function arrangePrevButton(bar:ControlBar, leftEdge:Number):Number {
		if (! bar.prevButton) return leftEdge;
		return arrangeToLeftEdge(leftEdge, getSpaceAfterWidget(bar.prevButton), bar.prevButton, bar);
	}
	
	private function arrangeNextButton(bar:ControlBar, leftEdge:Number):Number {
		if (! bar.nextButton) return leftEdge;
		return arrangeToLeftEdge(leftEdge, getSpaceAfterWidget(bar.nextButton), bar.nextButton, bar);
	}
	
	private function arrangeMenuButton(bar:ControlBar, edge:Number):Number {
		if (! bar.showMenu()) return edge;
		if (config.showScrubber())
			edge = arrangeToRightEdge(edge, getSpaceAfterWidget(bar.menuButton), bar.menuButton);
		else
			edge = arrangeToLeftEdge(edge, getSpaceAfterWidget(bar.menuButton), bar.menuButton, bar);
//		bar.menuButton._y = centeredY(bar.menuButton, bar);
		Util.centerClip(bar.menuButton, null, controlBarHeight(bar));
		return edge;
	}
	
	private function arrangeFullScreenButton(bar:ControlBar, edge:Number):Number {
		if (! bar.showFullScreenButton()) return edge;
		if (config.showScrubber())
			edge = arrangeToRightEdge(edge, getSpaceAfterWidget(bar.fullScreenButton), bar.fullScreenButton);
		else
			edge = arrangeToLeftEdge(edge, getSpaceAfterWidget(bar.fullScreenButton), bar.fullScreenButton, bar)
//		bar.fullScreenButton._y = centeredY(bar.fullScreenButton, bar);
		Util.centerClip(bar.fullScreenButton, null, controlBarHeight(bar));
		return edge;
	}
	
	private function centeredY(forClip:MovieClip, relativeTo:MovieClip):Number {
		return relativeTo._height / 2 - forClip._height / 2;
	}
	
	private function arrangeVolumeControl(bar:ControlBar, edge:Number):Number {
		if (! bar.showVolumeSlider()) return edge;
		bar.volume.setSize(getVolumeSliderWidth(), getVolumeSliderHeight());
		if (config.showScrubber())
			edge = arrangeToRightEdge(edge, getSpaceAfterWidget(bar.volume), bar.volume);
		else
			edge = arrangeToLeftEdge(edge, getSpaceAfterWidget(bar.volume), bar.volume, bar);
//		bar.volume._y = bar._height / 2 - bar.volume.__height / 2;
		Util.centerClip(bar.volume, null, controlBarHeight(bar));
		return edge;
	}
	
	private function arrangeMuteVolumeButton(bar:ControlBar, edge:Number):Number {
		if (! bar.showMuteVolumeButton()) return edge;
		if (config.showScrubber())
			edge = arrangeToRightEdge(edge, getSpaceAfterWidget(bar.muteVolumeButton), bar.muteVolumeButton);
		else
			edge = arrangeToLeftEdge(edge, getSpaceAfterWidget(bar.muteVolumeButton), bar.muteVolumeButton, bar);
//		bar.muteVolumeButton._y = centeredY(bar.muteVolumeButton, bar);
		Util.centerClip(bar.muteVolumeButton, null, controlBarHeight(bar));
		return edge;
	}
	
	private function arrangeProgressControl(bar:ControlBar, rightEdge:Number):Number {
		if (! bar.showScrubber()) return rightEdge;
		var prevButton:MovieClip = bar.lastButtonBeforeTracker();
		var space:Number = Math.max(getSpaceAfterWidget(prevButton), bar.tracker.mcDragger._width / 2 + 1);
		bar.tracker._visible = true;			
		bar.tracker._x = prevButton ? prevButton._x + prevButton._width + space : space;
//		bar.tracker._y = bar._height / 2 - bar.tracker.__height / 2;
		Util.centerClip(bar.tracker, null, controlBarHeight(bar));

		var trackerWidth:Number = rightEdge - bar.tracker._x - 2*getSpaceAfterWidget(bar.tracker); 
		bar.tracker.setSize(trackerWidth, getProgressBarHeight());
//		bar.tracker._y = centeredY(bar.tracker, bar);
		return rightEdge - getSpaceAfterWidget(bar.tracker) - bar.tracker._width;
	}
	
	public function arrangeThumbList(display:FLVDisplay, thumbList:MovieClip, controlBar:MovieClip):Void {
		if (! thumbList) return;
		if (display.getDisplayWidth() > thumbList._width)
			Util.centerClip(MovieClip(thumbList), display.getDisplayWidth());
		else
			thumbList._x = 0;
		thumbList._y = display.getDisplayHeight() + controlBarHeight(ControlBar(controlBar)) + PLAYLIST_TO_CONTROLS_GAP;
	}

	/*
	 * Array positions:
	 *        0
	 *   ----------
	 * 3 |        | 1
	 *   ----------
	 *        2
	 */
	public function getControlAreaMargins():Array {
		return [4, 10, 5, 10];
	}

	/*
	 * Array positions:
	 *        0
	 *   ----------
	 * 1 |        | 1
	 *   ----------
	 *        0
	 */
	public function getVideoAreaMargins():Array {
		return [0, 0];
	}
	
	private function arrangeClipNextToAnother(clip:MovieClip, another:MovieClip, spaceBetween:Number):Void {
		Util.arrangeClip(clip, another._x + another._width + (spaceBetween || 0), another._y, null, null);
	}
	
	private function arrangeToRightEdge(rightEdgePos:Number, spaceBetweenClips:Number, clip:MovieClip):Number {
		rightEdgePos = rightEdgePos - clip._width - spaceBetweenClips;
		clip._x = rightEdgePos;
		return rightEdgePos;
	}
	
	private function arrangeToLeftEdge(leftEdgePos:Number, spaceBetween:Number, clip:MovieClip, bar:ControlBar):Number {
		clip._x = leftEdgePos;
		clip._y = getControlAreaMargins()[0];
//		Util.centerClip(clip, undefined, bar._height);
		return leftEdgePos + clip._width + spaceBetween;
	}

	public function getTimePlayedFontColor() : Number {
		return 0xffffff;
	}

	public function fullScreenInit(display:FLVDisplay) : Void {
	}

	public function fullScreenExit(display:FLVDisplay):Void {
	}

	public function getEmailSendSuccessMessage(defaultMsg:String):String {
		return defaultMsg;
	}

	public function getEmailSendFailedMessage(defaultMsg:String):String {
		return defaultMsg;
	}

	public function getEmailSendInProgressMessage(defaultMsg:String):String {
		return defaultMsg;
	}

	public function getEmailSendMandatoryFieldMissing(defaultMsg:String):String {
		return defaultMsg;
	}
	
	public function arrangeLogo(videoOrigWidth:Number, display:FLVDisplay, hwFullScreen:Boolean):Void {
		if (! display.mcFPLogo) return;
		display.mcFPLogo._xscale = hwFullScreen ? getLogoHwScale(videoOrigWidth) : 100;
		display.mcFPLogo._yscale = hwFullScreen ? getLogoHwScale(videoOrigWidth) : 100;			
		display.mcFPLogo._x = display.getDisplayWidth() - display.mcFPLogo._width - (hwFullScreen ? 5 : 10);
		display.mcFPLogo._y = (hwFullScreen ? 5 : 10);
	}
	
	private function getLogoHwScale(videoOrigWidth:Number):Number {
		if (videoOrigWidth < 500) return 50;
		if (videoOrigWidth < 700) return 80;
		return 100;
	}
//
//	public function arrangeLogoForHwFullScreen(logo:MovieClip, videoArea:MovieClip):Void {
//		logo._xscale = 50;
//		logo._yscale = 50;			
//		logo._x = videoArea._width - logo._width - 30;
//		logo._y = 30;		
//	}

	public function setConfig(config:Config):Void {
		this.config = config;
	}
	
	public function getProgressBarHeight() : Number {
		return 10;
	}
	
	public function getVolumeSliderHeight() : Number {
		return 10;
	}
}