/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.AbstractDurationTrackingMediaControl;
import org.flowplayer.thumbs.SuggestionView;
import org.flowplayer.playlist.Clip;
import org.flowplayer.thumbs.SuggestionInfoLoader;
import flash.filters.DisplacementMapFilter;
import org.flowplayer.MediaControl;
import org.flowplayer.MediaView;
import org.flowplayer.Util;
import org.flowplayer.config.Config;
import org.flowplayer.MediaContext;
import org.flowplayer.thumbs.ThumbStrip;
import org.flowplayer.thumbs.Thumbnail;
import org.flowplayer.playlist.PlayList;

/**
 * @author Anssi
 */
class org.flowplayer.thumbs.SuggestionControl extends AbstractDurationTrackingMediaControl implements MediaControl {

	private var view:SuggestionView;
	private var loader:SuggestionInfoLoader;
	private var playList:PlayList;
	
	function SuggestionControl(playList:PlayList) {
		super();
		this.playList = playList;
	}
	
	public function attachView(view:MovieClip, mediaType:String):Void {
		this.view = SuggestionView(view);
		this.view.addEventListener("onArranged", this);
		this.view.addEventListener(ThumbStrip.THUMB_CLICKED_EVENT, this);
		this.view.addEventListener(SuggestionView.EVENT_REPLAY_CKICKED, this);
	}

	private function onStartBuffering(clip:Clip):Void {
		logger.debug("SuggestionControl.onStartBuffering()");
		if (clip.getSuggestionClips()) {
			logger.info("clip already has suggestions, displaying them");
			onSuggestionInfoLoaded(clip);
		} else {
			getInfoLoader().loadForClip(clip);
		}
	}

	private function getInfoLoader():SuggestionInfoLoader {
		if (! this.loader) {
			this.loader = new SuggestionInfoLoader();
			loader.addListener(SuggestionInfoLoader.EVENT_SUGGESTIONS_LOADED, this);
			loader.addListener(SuggestionInfoLoader.EVENT_SUGGESTIONS_LOAD_ERROR, this);
		}
		return this.loader;
	}
	
	public function onSuggestionInfoLoaded(clip:Object):Void {
		view.setClips(Clip(clip).getSuggestionClips());
	}

	public function onSuggestionInfoLoadError():Void {
		dispatchClipDone();
	}
	
	public function onArranged():Void {
		dispatchEventWithObj("onLoadBegin", this.clip);
		dispatchEventWithObj("onBufferFull", this);
	}
	
	public function onThumbClicked(clip:Clip):Void {
		logger.debug("SuggestionControl.onThumbClicked:: for clip " + clip);
		if (clip.getLinkURL())
			getURL(clip.getLinkURL(), clip.getLinkWindow());
		else
			changePlayList(clip);
	}

	private function changePlayList(suggestionToPlay:Clip) : Void {
		if (! suggestionToPlay) return;
		logger.debug("adding a suggestion to the playlist, suggetion is " + suggestionToPlay);
//		var current:Number = playList.currentIndex();
		var previousClip:Clip = playList.peekPrevious();
		if (previousClip) {
			playList.replace(previousClip, suggestionToPlay);
			playList.remove(this.clip);
		} else {
			playList.replace(playList.currentClip(), suggestionToPlay);
		}
		addFurtherSuggestion(suggestionToPlay);
		playList.toClip(suggestionToPlay, true);
		logger.debug("current in playlist is " + playList.currentClip());
		dispatchEventWithObj("onPlayListChangedStartPlay", playList);
	}
	
	private function addFurtherSuggestion(forClip:Clip):Void {
		if (! clip.getSuggestionsInfoUrl()) return;
		playList.addClipAfterAnother(Clip.createSuggestionInfo(undefined, forClip.getSuggestionsInfoUrl()), forClip);
	}
	
	private function onReplayClicked():Void {
		replayPreviousClip();
	}

	private function replayPreviousClip():Void {
		var previous:Clip = playList.prev();
		dispatchEvent("onPlayInitiated");
	}

	public function setSound(sound : Sound) : Void {
	}


	public function setViewVisibility():Boolean {
		return false;
	}

	// override following because we really don't need time tracking for now
	private function createTimeUpdateInterval():Void {
	}		

	public function allowCenterViewOnLoadBegin() : Boolean {
		return true;
	}
	
	public function rewind():Void {
	}
}