﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowlib.HorizontalScrollBar;
import org.flowplayer.ClipDescriptor;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.thumbs.Thumbnail;
import org.flowplayer.thumbs.ThumbStrip;

class org.flowplayer.thumbs.ThumbList extends NewAbstractMovieClip {
	
	private var holder:ThumbStrip;
	private var scrollBar:HorizontalScrollBar;
	private var thumbClips:Array;
	private static var THUMB_SPACING:Number = 5;
	
	public function ThumbList() {
		super();
		this.visible = false;
	}
	
	public function setThumbnails(thumbs:Array):Void {
		logger.info("ThumbList.setThumbnails");
		holder.setThumbnails(thumbs);
	}
	
	private function getChildDescriptors():Array {
		return [ 
			new ClipDescriptor("ThumbStrip", "holder", null),
			new ClipDescriptor("XScrollBar", "scrollBar", null)
		];
	}
	
	private function allChildrenCreated(clips:Array):Void {
		logger.info("ThumbList.allChildrenCreated");
		this.holder = clips[0];
		holder.addEventListener("onArranged", this);
		
		this.scrollBar = clips[1];
		scrollBar.addEventListener("onScroll", this);
	}
	

	function arrange():Void {
		holder._x = 0;
		if (__width > holder._width) {
			scrollBar._visible = false;
			scrollBar._y = 0;
		} else {
			scrollBar._visible = true;
			scrollBar.setScrollLengths(__width, holder._width, 20);
			scrollBar._x = 0;
			scrollBar._y = holder._y + holder._height + 5;
			logger.info("Setting scrollbar width to " + __width);
		}
		this._height = holder._height + (scrollBar._visible ? scrollBar._height : 0);
		this.visible = true;
	}
	
	function onArranged(thumbsChanged:Boolean):Void {
		logger.debug("ThumbList received onArranged()");
		if (thumbsChanged) {
			arrange();
			dispatchEvent("onArranged", this);
		}
	}
		
	public function onScroll():Void {
		var position:Number = scrollBar.getScrollPosition();
		holder._x = - position * (holder._width - __width);
	}
	
	public function onThumbClicked(thumb:Thumbnail):Void {
		// dispatch to my listeners
		dispatchEvent(ThumbStrip.THUMB_CLICKED_EVENT, thumb);
		if (getConfig().thumbsOnFLV())
			holder.highlightThumbOnTime(thumb.getTime());
	}
	
	public function onCuePointWithThumb(time:Number):Void {
		logger.info(this._name + " onCuePointWithThumb, time " + time + ", holder " + holder);
		var activeThumb:MovieClip = holder.highlightThumbOnTime(time);
	}
	
	public function setThumbsPerRow(numOfThumbsPerRow:Number):Void {
		holder.setThumbsPerRow(numOfThumbsPerRow);
	}

}