﻿/*    
 *    Copyright 2008 Anssi Piirainen
 *
 *    This file is part of FlowPlayer.
 *
 *    FlowPlayer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Foobar is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with FlowPlayer.  If not, see <http://www.gnu.org/licenses/>.
 */
import org.flowplayer.ClipDescriptor;
import org.flowlib.NewAbstractMovieClip;
import org.flowplayer.thumbs.ThumbList;
import org.flowplayer.thumbs.Thumbnail;
import org.flowplayer.thumbs.ThumbnailView;

/**
 * @author anssi
 */
class org.flowplayer.thumbs.ThumbStrip extends NewAbstractMovieClip {
	public static var THUMB_CLICKED_EVENT:String = "onThumbClicked";
	private var numThumbsPerRow:Number;
	private var thumbsLoaded:Number = 0;
	private var thumbNails:Array;
	private var thumbClips:Array;
	private var currentlyHighlighted:ThumbnailView;
	private static var THUMB_SPACING:Number = 0;
	private var thumbClipsByTime : Array;

	
	public function ThumbStrip() {
		super();
//		this.visible = false;
	}
	
	public function setThumbnails(thumbs:Array):Void {
		if (sameThumbs(this.thumbNails, thumbs)) {
			logger.debug("already showing the same thumbs, dispatching onArrange");
			logger.debug(this.thumbNails);
			logger.debug(thumbs);
			dispatchEvent("onArranged");
			return;
		}
		this.thumbNails = thumbs;
		removeThumbnails();
		loadThumbnails(thumbs);		
	}

	private function removeThumbnails() : Void {
		for (var i : Number = 0; i < thumbClips.length; i++) {
			var thumb:MovieClip = thumbClips[i];
			thumb.removeMovieClip();
		}
		thumbsLoaded = 0;
	}
	
	private function loadThumbnails(thumbs:Array):Void {
		logger.info(this._name + ".loadThumbnails :");
		logger.info(thumbs);
		var descriptors:Array = new Array();
		for (var i : Number = 0; i < thumbs.length; i++) {
			var thumb:Thumbnail = thumbs[i];
			descriptors.push(new ClipDescriptor("ThumbnailView", "thumb" + i, null));
		}
		loadClips(descriptors);
	}
	
	private function allChildrenCreated(clips:Array):Void {
		this.thumbClips = clips;
		this.thumbClipsByTime = new Array();
		logger.info("number of thumb clips " + clips.length);
		
		for (var i : Number = 0; i < clips.length; i++) {
			var thumb:ThumbnailView = clips[i];
			var thumbNail:Thumbnail = thumbNails[i];
			thumbClipsByTime[thumbNail.getTime().toString()] = thumb;
			thumb.loadThumb(thumbNails[i]);
			logger.debug("loaded thumb for " + thumbNails[i] + ", visible = " + thumb._visible);
			thumb.addEventListener("onArranged", this);
			thumb.addEventListener(ThumbStrip.THUMB_CLICKED_EVENT, this._parent);
		}
	}
	

	function arrange():Void {
	}
	
	private function arrangeThumbs():Void {
		var xpos:Number = 0;
		var ypos:Number = 0;
		for (var i : Number = 0; i < thumbClips.length; i++) {
			var clip:ThumbnailView = thumbClips[i];
			logger.debug("arranging " + clip._name + ", visible = " + clip._visible);
			if (rowFilled(i)) {
				xpos = 0;
				ypos += clip._height + THUMB_SPACING; 
			}
			clip._x = xpos;
			clip._y = ypos;
			clip.visible = true;
			xpos += clip._width + THUMB_SPACING;
		}
		dispatchEvent("onArranged", true);
	}
	
	private function rowFilled(thumbsFilled:Number):Boolean {
		if (! numThumbsPerRow) return false;
		if (thumbsFilled <= 0) return false;
		return thumbsFilled % numThumbsPerRow == 0;
	}
	
	function onArranged():Void {
		if (++thumbsLoaded == thumbClips.length) {
			arrangeThumbs();
		} else {
			logger.debug("number of thumbs loaded " + thumbsLoaded + ", total thumbs " + thumbClips.length);
		}
	}
	
	public function highlightThumbOnTime(time:Number):MovieClip {
		logger.debug(this._name + "highlightThumbOnIndex, time " + time);
		if (currentlyHighlighted != undefined)
			currentlyHighlighted.stopHighlight();
			
		currentlyHighlighted = thumbClipsByTime[time];
		currentlyHighlighted.startHighlight();
		return currentlyHighlighted;
	}

	private function sameThumbs(thumbs1 : Array, thumbs2 : Array):Boolean {
		if (thumbs1.length != thumbs2.length) return false;
		for (var i : Number = 0; i < thumbs1.length; i++) {
			var thumb1:Thumbnail = thumbs1[i];
			var thumb2:Thumbnail = thumbs2[i];
			if (thumb1.getUrl() != thumb2.getUrl())
				return false;
		}
		return true;
	}

	public function setThumbsPerRow(numThumbsPerRow:Number):Void {
		this.numThumbsPerRow = numThumbsPerRow;
	}

}