import org.flowplayer.playlist.Clip;
import org.flowplayer.playlist.PlayList;

import asunit.framework.TestCase;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;

/**
 * @author anssi
 */
class org.flowplayer.playlist.PlayListTest extends TestCase {

	private var logger:Logger;
	
	public function setUp():Void {
		logger = new Logger();
		logger.addPublisher(new ConsolePublisher());
	}

	public function testGetClipTypes():Void {
		var clips:Array = new Array();

		clips.push({ url : "Thumb1.jpg", duration : 3 });
		clips.push({ url : "clip1.flv", duration : 5 });
		clips.push({ url : "music.mp3", duration : 7 });
		
		var playList:PlayList = PlayList.fromObjectsWithProps("path", clips);
		
		var types:Array = playList.getClipTypes();
		logger.debug(types);
		assertTrue(types[0] != undefined);
		assertTrue(types[1] != undefined);
		assertTrue(types[2] != undefined);
		assertEquals("wrong types array length", 3, types.length);
	}
	
	public function testFromObjectsWithPropsCombimesAdjacentStreams():Void {
		var clips:Array = new Array();

		clips.push({ url : "Thumb1.jpg", duration : 3 });
		clips.push({ url : "clip1.flv", duration : 5 });
		clips.push({ url : "clip2.flv", duration : 5 });
		clips.push({ url : "clip3.flv", duration : 5 });
		clips.push({ url : "music.mp3", duration : 7 });
		clips.push({ url : "clip4.flv", duration : 5 });
		clips.push({ url : "clip5.flv", duration : 5 });
		
		var playList:PlayList = PlayList.fromObjectsWithProps("path", clips, false, null, true);
		
		var clip:Clip = playList.currentClip();
		assertEquals("path/Thumb1.jpg", clip.getURL());
		assertEquals(0, clip.getChildren().length);
		
		clip = playList.next();
		assertEquals("path/clip1.flv", clip.getURL());
		var children:Array = clip.getChildren();
		logger.debug(children);
		assertEquals("wrong number of child clips for clip1.flv: " + children.length, 2, children.length);
		assertEquals("wrong url: " + children[0].getURL(), "path/clip2.flv", children[0].getURL());
		assertEquals("wrong url: " + children[1].getURL(), "path/clip3.flv", children[1].getURL());
		
		playList.prev();
		playList.next();
		
		clip = playList.next();
		assertEquals("path/music.mp3", clip.getURL());
		children = clip.getChildren();
		logger.debug(children);
		assertEquals("wrong number of child clips for music.mp3: " + children.length, 0, children.length);
		
		clip = playList.next();
		assertEquals("path/clip4.flv", clip.getURL());
		children = clip.getChildren();
		logger.debug(children);
		assertEquals("wrong number of child clips for clip4.flv: " + children.length, 1, children.length);
		assertEquals("wrong url: " + children[0].getURL(), "path/clip5.flv", children[0].getURL());
	}
}